# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import Callbacks as Callbacks
import PyFnAttribute as FnAttribute
import NodegraphAPI_cmodule as NodegraphAPI_cmodule
import PyFnAttribute
import Utils as Utils
import typing
from . import BypassParameterManager as BypassParameterManager, CallbackTypes as CallbackTypes, Constants as Constants, DynamicParameters as DynamicParameters, ExpressionModule as ExpressionModule, Flavor as Flavor, GroupStack as GroupStack, InteractiveRenderFilters as InteractiveRenderFilters, LiveGroup as LiveGroup, NodeDelegateManager as NodeDelegateManager, NodeExtensions as NodeExtensions, NodeGraphViewExtensions as NodeGraphViewExtensions, NodeNameFromParamManager as NodeNameFromParamManager, NodeXmlIO as NodeXmlIO, NodegraphChangedEvents as NodegraphChangedEvents, NodegraphGlobals as NodegraphGlobals, ParameterValueOption as ParameterValueOption, RenderFilter as RenderFilter, RenderScript as RenderScript, StructuredPorts as StructuredPorts, UserParameters as UserParameters, Util as Util, Version as Version, Xio as Xio
from NodegraphAPI.DynamicParameters import CreateGroupParameter as CreateGroupParameter, CreateNumberParameter as CreateNumberParameter, CreateStringParameter as CreateStringParameter, DeleteParameter as DeleteParameter
from NodegraphAPI.ExpressionModule import FCurveLookup as FCurveLookup, GetExpressionGlobalValue as GetExpressionGlobalValue, SetExpressionGlobalValue as SetExpressionGlobalValue, SetupExpressionGlobalScopeFromRootNode as SetupExpressionGlobalScopeFromRootNode
from NodegraphAPI.Flavor import AddNodeFlavor as AddNodeFlavor, ClearFlavorNodes as ClearFlavorNodes, GetAllFlavors as GetAllFlavors, GetFlavorNodes as GetFlavorNodes, GetNodeFlavors as GetNodeFlavors, NodeMatchesFlavors as NodeMatchesFlavors, RemoveNodeFlavor as RemoveNodeFlavor
from NodegraphAPI.LiveGroup import AssetPublishing as AssetPublishing, CalculateLiveGroupDepth as CalculateLiveGroupDepth, CanBeDifferentFromSource as CanBeDifferentFromSource, ConvertGroupToLiveGroup as ConvertGroupToLiveGroup, IsLiveGroupCachingEnabled as IsLiveGroupCachingEnabled, IsLiveGroupLoadingEnabled as IsLiveGroupLoadingEnabled, LiveGroupNode as LiveGroupNode, LoadAllLiveGroups as LoadAllLiveGroups, LockAllLiveGroups as LockAllLiveGroups, MakeAllLiveGroupsEditable as MakeAllLiveGroupsEditable, MakeAllParentLiveGroupsEditable as MakeAllParentLiveGroupsEditable, SetLiveGroupCachingEnabled as SetLiveGroupCachingEnabled, SetLiveGroupLoadingEnabled as SetLiveGroupLoadingEnabled, UpdateAllLiveGroupSources as UpdateAllLiveGroupSources
from NodegraphAPI.NodeExtensions import DeleteNodeShapeAttr as DeleteNodeShapeAttr, GetAllActiveAttributeEditorNodes as GetAllActiveAttributeEditorNodes, GetAllEditedNodes as GetAllEditedNodes, GetAllFloatingNodes as GetAllFloatingNodes, GetAllSelectedNodes as GetAllSelectedNodes, GetAllSelectedNodesAndParent as GetAllSelectedNodesAndParent, GetAllThumbnailEnabledNodes as GetAllThumbnailEnabledNodes, GetMenuPath as GetMenuPath, GetNodeComment as GetNodeComment, GetNodeModTime as GetNodeModTime, GetNodeParameter as GetNodeParameter, GetNodePosition as GetNodePosition, GetNodeShapeAttr as GetNodeShapeAttr, GetNodeShapeAttrs as GetNodeShapeAttrs, GetViewNode as GetViewNode, GetViewNodes as GetViewNodes, GetViewPortPosition as GetViewPortPosition, IsAttributeEditorNodeActive as IsAttributeEditorNodeActive, IsNodeEdited as IsNodeEdited, IsNodeFloating as IsNodeFloating, IsNodeHidden as IsNodeHidden, IsNodeLockedByParents as IsNodeLockedByParents, IsNodeSelected as IsNodeSelected, IsNodeThumbnailEnabled as IsNodeThumbnailEnabled, IsNodeViewed as IsNodeViewed, Register_NewNode_CB as Register_NewNode_CB, SetAllSelectedNodes as SetAllSelectedNodes, SetAttributeEditorNodeActive as SetAttributeEditorNodeActive, SetMenuPath as SetMenuPath, SetNodeComment as SetNodeComment, SetNodeEdited as SetNodeEdited, SetNodeFloating as SetNodeFloating, SetNodeHidden as SetNodeHidden, SetNodeModTime as SetNodeModTime, SetNodeParameter as SetNodeParameter, SetNodePosition as SetNodePosition, SetNodeSelected as SetNodeSelected, SetNodeShapeAttr as SetNodeShapeAttr, SetNodeShapeNodeAttrs as SetNodeShapeNodeAttrs, SetNodeThumbnailEnabled as SetNodeThumbnailEnabled, SetNodeViewed as SetNodeViewed, SetViewPortPosition as SetViewPortPosition
from NodegraphAPI.NodeXmlIO import BuildNodegraphXmlIO as BuildNodegraphXmlIO, BuildNodesXmlIO as BuildNodesXmlIO, GetUserNodesXmlRootAttrs as GetUserNodesXmlRootAttrs, LoadElementsFromFile as LoadElementsFromFile, LoadElementsFromString as LoadElementsFromString, ParseNodegraphXmlIO as ParseNodegraphXmlIO, ParseNodesXmlIO as ParseNodesXmlIO, SetUserNodesXmlRootAttrs as SetUserNodesXmlRootAttrs
from NodegraphAPI.NodegraphGlobals import CheckNodeType as CheckNodeType, GetAutoKeyAll as GetAutoKeyAll, GetCurrentTime as GetCurrentTime, GetInTime as GetInTime, GetKatanaSceneName as GetKatanaSceneName, GetNodegraphVersionString as GetNodegraphVersionString, GetOriginalProjectAssetID as GetOriginalProjectAssetID, GetOriginalSourceFile as GetOriginalSourceFile, GetOutTime as GetOutTime, GetProjectAssetID as GetProjectAssetID, GetProjectDir as GetProjectDir, GetProjectFile as GetProjectFile, GetRootNode as GetRootNode, GetSourceFile as GetSourceFile, GetTimeIncrement as GetTimeIncrement, GetWorkingInTime as GetWorkingInTime, GetWorkingOutTime as GetWorkingOutTime, IsLoading as IsLoading, IsLoadingAsync as IsLoadingAsync, LoadXmlFromFile as LoadXmlFromFile, LoadXmlFromString as LoadXmlFromString, NeedsRedraw as NeedsRedraw, NewNodegraph as NewNodegraph, ResetAllFloatingCurves as ResetAllFloatingCurves, SetAutoKeyAll as SetAutoKeyAll, SetCurrentTime as SetCurrentTime, SetInTime as SetInTime, SetKatanaSceneName as SetKatanaSceneName, SetNeedsRedraw as SetNeedsRedraw, SetOriginalProjectAssetID as SetOriginalProjectAssetID, SetOriginalSourceFile as SetOriginalSourceFile, SetOutTime as SetOutTime, SetProjectAssetID as SetProjectAssetID, SetRootNode as SetRootNode, SetSourceFile as SetSourceFile, SetTimeIncrement as SetTimeIncrement, SetWorkingInTime as SetWorkingInTime, SetWorkingOutTime as SetWorkingOutTime, UpdatePluginsFromGlobals as UpdatePluginsFromGlobals, WriteKatanaFile as WriteKatanaFile, WriteKatanaString as WriteKatanaString
from NodegraphAPI.PythonRenderScript import PythonRenderScript as PythonRenderScript
from NodegraphAPI.SuperTool import RegisterSuperTool as RegisterSuperTool, SuperTool as SuperTool
from NodegraphAPI_cmodule import Context as Context, GetAllExpressionedParameters as GetAllExpressionedParameters, GetExpressionReferences as GetExpressionReferences, GetNodeGraphContext as GetNodeGraphContext, GraphState as GraphState, GraphStateBuilder as GraphStateBuilder, GroupNode as GroupNode, Node as Node, Parameter as Parameter, Port as Port, PythonGroupNode as PythonGroupNode, PythonNode as PythonNode
from _typeshed import Incomplete
from typing import Set, Tuple

NodeGraphContextParameter: str
RTLD_GLOBAL: int
RTLD_NOW: int
_IsInSynchronousContext: bool
_isNodegraphLoading: bool
useFlags: bool

def AreEventsEnabled(): ...
def AsynchronousContext(): ...
def BuildAttrFromGroupParameter(parameter: Parameter, graphState: typing.Optional[GraphState] = ..., multisampleDefault: bool = ..., numberAttrTypeDefault: type[PyFnAttribute.FloatAttribute] = ..., includeEmptyGroups: bool = ..., groupInherit: bool = ...) -> PyFnAttribute.GroupAttribute | None: ...
def BuildAttrFromNumberParameter(parameter: Parameter, graphState: typing.Optional[GraphState] = ..., multisample: bool = ..., attrType: type[PyFnAttribute.FloatAttribute] = ...) -> attrType | None: ...
def BuildAttrFromStringParameter(parameter: Parameter, graphState: typing.Optional[GraphState] = ..., multisample: bool = ...) -> PyFnAttribute.StringAttribute | None: ...
def BuildDoubleAttrFromNumberParameter(parameter: Parameter, graphState: typing.Optional[GraphState] = ..., multisample: bool = ...) -> PyFnAttribute.DoubleAttribute | None: ...
def BuildFloatAttrFromNumberParameter(parameter: Parameter, graphState: typing.Optional[GraphState] = ..., multisample: bool = ...) -> PyFnAttribute.FloatAttribute | None: ...
def BuildIntAttrFromNumberParameter(parameter: Parameter, graphState: typing.Optional[GraphState] = ..., multisample: bool = ...) -> PyFnAttribute.IntAttribute | None: ...
def CreateNode(type: str, parent: Incomplete | None = ...) -> Node: ...
def GetAllNodes(includeDeleted: bool = ..., sortByName: bool = ...) -> list[Node]: ...
def GetAllNodesByType(nodeType: str, includeDeleted: bool = ..., sortByName: bool = ...) -> list[Node]: ...
def GetCurrentGraphState(**kwargs): ...
def GetGraphState(timeval: float, **kwargs) -> GraphState: ...
def GetGraphStateAtPort(searchPort: Port, startPort: Port, startGraphState: GraphState) -> GraphState | None: ...
def GetNode(nodeName: str) -> Node | None: ...
def GetNodeContext(node: Node): ...
def GetNodeTypes() -> list[str]: ...
def GetSampleTimes(shutterOpen: float, shutterClose: float, maxTimeSamples: int, useSinglePrecision: bool = ...) -> tuple[float, ...]: ...
def GetSampleTimesFromGraphState(graphState: GraphState, useSinglePrecision: bool = ...) -> tuple[float, ...]: ...
def GetSingleSampleTimeFromGraphState(graphState: GraphState) -> float: ...
def InteractiveContext(): ...
def IsInSynchronousContext() -> bool: ...
def IsParameterConstant(parameter: Parameter) -> bool: ...
def NonInteractiveContext(): ...
def PopExpressionCache(): ...
def PushExpressionCache(): ...
def RegisterExtension(ext): ...
def RegisterPythonGroupType(type: str, ctor): ...
def RegisterPythonNodeFactory(type: str, factory: typing.Callable): ...
def RegisterPythonNodeType(type: str, ctor): ...
def SetEventsEnabled(state: int): ...
def StackedLocalGraphState(graphState: GraphState) -> None: ...
def SynchronousContext(): ...
def UnlockedNode(): ...
def __loadBeginCallback(eventType, eventID, **kwargs): ...
def __loadEndCallback(eventType, eventID, **kwargs): ...
def __nodeCreateCallback(eventType, eventID, **kwargs): ...
def __nodeDeleteCallback(eventType, eventID, **kwargs): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import PyQt5.QtCore
import PyQt5.QtGui
import PyQt5.QtWidgets
import PyQt5.QtCore as QtCore
import PyQt5.QtGui as QtGui
import PyQt5.QtWidgets as QtWidgets
import QT4Widgets.TreeWidgetUtil as TreeWidgetUtil
from _typeshed import Incomplete
from typing import ClassVar, Set, Tuple

class CallbackRecord:
    def __init__(self) -> None: ...
    def accept(self): ...
    def ignore(self): ...
    def isAccepted(self): ...

class SortableTreeWidget(PyQt5.QtWidgets.QTreeWidget):
    class _SortableTreeWidget__ContentsPosStore:
        def __init__(self, w, restoreOnDelete: bool = ...) -> None: ...
        def get(self): ...
        def restore(self): ...
        def __del__(self) -> None: ...
    aboutToDrag: ClassVar[PyQt5.QtCore.pyqtSignal] = ...
    contextMenuEventSignal: ClassVar[PyQt5.QtCore.pyqtSignal] = ...
    dragEnterEventSignal: ClassVar[PyQt5.QtCore.pyqtSignal] = ...
    dragMoveEventSignal: ClassVar[PyQt5.QtCore.pyqtSignal] = ...
    dropEventSignal: ClassVar[PyQt5.QtCore.pyqtSignal] = ...
    focusInEventSignal: ClassVar[PyQt5.QtCore.pyqtSignal] = ...
    keyPressEventSignal: ClassVar[PyQt5.QtCore.pyqtSignal] = ...
    mouseMoveEventSignal: ClassVar[PyQt5.QtCore.pyqtSignal] = ...
    mousePressEventSignal: ClassVar[PyQt5.QtCore.pyqtSignal] = ...
    mouseReleaseEventSignal: ClassVar[PyQt5.QtCore.pyqtSignal] = ...
    rawDragMoveEvent: ClassVar[PyQt5.QtCore.pyqtSignal] = ...
    rawDropEvent: ClassVar[PyQt5.QtCore.pyqtSignal] = ...
    def __init__(self, parent, delegate: Incomplete | None = ...) -> None: ...
    def _SortableTreeWidget__getExpandedAndSelectionTables(self, parentPath, delimiter, items, selectedTable, expandedTable): ...
    def _SortableTreeWidget__itemExpandedOrCollapsed(self, *args, **kwargs): ...
    def _SortableTreeWidget__lastItem(self) -> PyQt5.QtWidgets.QTreeWidgetItem | None: ...
    def _SortableTreeWidget__processDrag(self, event, handleScrolling: bool = ...): ...
    def _SortableTreeWidget__startDrag(self): ...
    def commitData(self, editor: PyQt5.QtWidgets.QWidget): ...
    def contextMenuEvent(self, event: PyQt5.QtGui.QContextMenuEvent): ...
    def customEvent(self, event): ...
    def dragEnterEvent(self, event): ...
    def dragLeaveEvent(self, event): ...
    def dragMoveEvent(self, event): ...
    def dropEvent(self, event): ...
    def edit(self, index: PyQt5.QtCore.QModelIndex, trigger: PyQt5.QtWidgets.QAbstractItemView.EditTrigger, event: PyQt5.QtCore.QEvent) -> bool: ...
    def findItemLocalIndex(self, item): ...
    def getContextMenuHelper(self, event): ...
    def getDragItems(self): ...
    def getExpandedAndSelectionTables(self, delimiter: str = ...): ...
    def getFullHeight(self): ...
    def getFullWidth(self): ...
    def getItemFromLocalIndex(self, parent, index): ...
    def getPositionStore(self, restoreOnDelete: bool = ...): ...
    def getUpdateSuppressor(self): ...
    def isDraggable(self): ...
    def isItemDraggable(self, item): ...
    def isMultiDragEnabled(self): ...
    def keyPressEvent(self, event: PyQt5.QtGui.QKeyEvent): ...
    def mouseMoveEvent(self, event: PyQt5.QtGui.QMouseEvent): ...
    def mousePressEvent(self, event): ...
    def mouseReleaseEvent(self, event): ...
    def setDraggable(self, state): ...
    def setMultiDragEnabled(self, state): ...

class SortableTreeWidgetItem(PyQt5.QtWidgets.QTreeWidgetItem):
    def __init__(self, parent, text: str = ..., data: Incomplete | None = ..., showFocus: bool = ...) -> None: ...
    def getHiliteColor(self): ...
    def getItemData(self): ...
    def indent(self): ...
    def isDraggable(self): ...
    def path(self, column: int = ...): ...
    def setExpanded(self, expand): ...
    def setHiliteColor(self, color): ...
    def setItemData(self, data): ...
    def __hash__(self) -> int: ...
    def __lt__(self, other) -> bool: ...

class SortableTreeWidgetItemDelegate(PyQt5.QtWidgets.QStyledItemDelegate):
    def initStyleOption(self, option: PyQt5.QtWidgets.QStyleOptionViewItem, index: PyQt5.QtCore.QModelIndex): ...
    def textColor(self, option: PyQt5.QtWidgets.QStyleOptionViewItem) -> PyQt5.QtGui.QColor: ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import UI4.Util.IconManager as IconManager
import PyQt5.QtCore
import QT4Widgets as QT4Widgets
import PyQt5.QtCore as QtCore
import PyQt5.QtGui as QtGui
import PyQt5.QtWidgets as QtWidgets
from QT4Widgets.FilterablePopupButton import FilterablePopupButton
from typing import ClassVar, Set, Tuple

class NonexclusiveCheckboxPopup(FilterablePopupButton):
    valuesChanged: ClassVar[PyQt5.QtCore.pyqtSignal] = ...
    def __init__(self, parent, emptyLabel: str = ..., showValues: bool = ..., nameCheckable: bool = ...) -> None: ...
    def _NonexclusiveCheckboxPopup__addToValues(self, value): ...
    def _NonexclusiveCheckboxPopup__itemChosen(self, name, meta): ...
    def _NonexclusiveCheckboxPopup__popupClosed(self): ...
    def _NonexclusiveCheckboxPopup__removeFromValues(self, value): ...
    def _NonexclusiveCheckboxPopup__setItemCheck(self, item): ...
    def _NonexclusiveCheckboxPopup__treeCheckboxItem(self, event): ...
    def _NonexclusiveCheckboxPopup__treeMouseMoveEvent(self, event): ...
    def _NonexclusiveCheckboxPopup__treeMousePressEvent(self, event): ...
    def _NonexclusiveCheckboxPopup__treeMouseReleaseEvent(self, event): ...
    def addItem(self, *args, **kwds): ...
    def clear(self): ...
    def getItemNames(self): ...
    def getValues(self): ...
    def setActiveColor(self, c): ...
    def setDragToToggle(self, state): ...
    def setEmptyLabel(self, label): ...
    def setNoActiveColor(self): ...
    def setValues(self, values, updateItems: bool = ..., sendSignal: bool = ..., force: bool = ...): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import PyQt5.QtWidgets
import PyQt5.QtWidgets as QtWidgets
from typing import Set, Tuple

class DoubleSpinBox(PyQt5.QtWidgets.QDoubleSpinBox):
    def __init__(self, parent) -> None: ...
    def mouseReleaseEvent(self, event): ...
    def stepBy(self, steps): ...
    def wheelEvent(self, event): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import PyQt5.QtGui
import PyQt5.QtWidgets
import QT4Widgets as QT4Widgets
import QT4Widgets.SortableTreeWidget
import PyQt5.QtCore as QtCore
import PyQt5.QtWidgets as QtWidgets
import UI4.Util.ScenegraphIconManager as ScenegraphIconManager
import Utils as Utils
from typing import ClassVar, Set, Tuple

class AttributeDataTreeWidgetItem(BaseTreeWidgetItem):
    AttributeName: ClassVar[str] = ...
    LocationType: ClassVar[str] = ...
    @classmethod
    def ConfigureDrag(cls, draggedItems: list[LocationTreeWidgetItem], drag: PyQt5.QtGui.QDrag): ...
    @classmethod
    def getAttributeName(cls) -> str: ...
    @classmethod
    def getLocationType(cls) -> str: ...
    def getSortID(self): ...
    def shouldDrawGreyForeground(self) -> bool: ...
    def updateDisplayData(self): ...

class BaseTreeWidgetItem(QT4Widgets.SortableTreeWidget.SortableTreeWidgetItem):
    def __init__(self, parent: PyQt5.QtWidgets.QTreeWidgetItem | None, handle, topLevelHandle) -> None: ...
    @classmethod
    def ConfigureDrag(cls, draggedItems: list[BaseTreeWidgetItem], drag: PyQt5.QtGui.QDrag): ...
    def attributeDataChildCount(self): ...
    def getChildItems(self, visibleOnly: bool = ..., locationsOnly: bool = ..., allDescendants: bool = ...) -> list[LocationTreeWidgetItem]: ...
    def getChildSortID(self, childName): ...
    def getHandle(self): ...
    def getLocationPath(self): ...
    def getNumChildSortIDs(self) -> int: ...
    def getSortID(self) -> int: ...
    def getTopLevelHandle(self): ...
    def getTopLevelIndex(self): ...
    def getTopLevelLocationPath(self): ...
    def inheritSortID(self): ...
    def isAttributeDataItem(self) -> bool: ...
    @classmethod
    def isAttributeDataType(cls) -> bool: ...
    def isLocationItem(self) -> bool: ...
    @classmethod
    def isLocationType(cls) -> bool: ...
    def setAttributeDataChildren(self, attributeDataChildHandles: dict): ...
    def setChildSortOrder(self, childNames: list[str], updateChildren: bool = ...): ...
    def setHandle(self, handle): ...
    def setSortID(self, sortId): ...
    def setTopLevelHandle(self, topLevelHandle): ...
    def setTopLevelIndex(self, topLevelIndex): ...
    def updateChildIndicator(self): ...
    def __lt__(self, item) -> bool: ...

class LocationTreeWidgetItem(BaseTreeWidgetItem):
    TYPE_COLUMN: ClassVar[int] = ...
    @classmethod
    def ConfigureDrag(cls, draggedItems: list[LocationTreeWidgetItem], drag: PyQt5.QtGui.QDrag): ...
    def getLocationType(self) -> str: ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import NodegraphAPI as NodegraphAPI
import Nodes2DAPI as Nodes2DAPI
import Nodes3DAPI.RenderNodeUtil as RenderNodeUtil
from _typeshed import Incomplete
from typing import Set, Tuple

def __nodegraphExpressionGetRenderLocation(nodeParamRef, output: int = ..., frame: Incomplete | None = ...): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import NodegraphAPI
import typing
from typing import Any, ClassVar, Set, Tuple, overload

__cleanupNodeClass: PyCapsule
__cleanupNotify: PyCapsule
__cleanupPortClass: PyCapsule
__cleanupPythonNodeClasses: PyCapsule

class Context:
    Legacy: ClassVar[str] = ...
    NetworkMaterial: ClassVar[str] = ...
    NodeGraphContextParameter: ClassVar[str] = ...
    def __init__(self, *args, **kwargs) -> None: ...
    @classmethod
    def GetNodeGraphContext(cls, node: NodegraphAPI.Node) -> str: ...

class Flavor:
    RendererAgnostic: ClassVar[str] = ...
    def __init__(self, *args, **kwargs) -> None: ...

class GraphState:
    def __init__(self) -> None: ...
    def edit(self) -> GraphStateBuilder: ...
    def getDynamicEntry(self, path: str) -> Any: ...
    def getDynamicHash(self) -> int: ...
    def getHash(self) -> int: ...
    def getMaxTimeSamples(self) -> int: ...
    def getOpSystemArgs(self) -> GroupAttribute: ...
    def getROI(self) -> tuple: ...
    def getRenderMethodName(self) -> str: ...
    def getSampleRateX(self) -> float: ...
    def getSampleRateY(self) -> float: ...
    def getShutterClose(self) -> float: ...
    def getShutterOpen(self) -> float: ...
    def getStaticEntry(self, path: str) -> Any: ...
    def getStaticHash(self) -> int: ...
    def getTime(self) -> float: ...
    def getView(self) -> str: ...
    def isDiskCachingAllowed(self) -> bool: ...
    def isExternalRenderAllowed(self) -> bool: ...
    def isHotRender(self) -> bool: ...
    def isROIAbsolute(self) -> bool: ...
    def isTaskCachingIgnored(self) -> bool: ...
    def matchVariableWithCEL(self, variableName: str, celStatement: str) -> bool: ...
    def __eq__(self, other: object) -> bool: ...
    def __hash__(self) -> int: ...
    def __lt__(self, other: object) -> bool: ...

class GraphStateBuilder:
    def __init__(self) -> None: ...
    @overload
    def build(self) -> GraphState: ...
    @overload
    def build(self) -> Any: ...
    def deleteDynamicEntry(self, path: str) -> GraphStateBuilder: ...
    def deleteStaticEntry(self, path: str) -> GraphStateBuilder: ...
    def setDiskCachingAllowed(self, allowCaching: bool) -> GraphStateBuilder: ...
    @overload
    def setDynamicEntry(self, path: str, attr) -> GraphStateBuilder: ...
    @overload
    def setDynamicEntry(self, path: str, value: int) -> GraphStateBuilder: ...
    @overload
    def setDynamicEntry(self, path: str, value: float) -> GraphStateBuilder: ...
    @overload
    def setDynamicEntry(self, path: str, value: str) -> GraphStateBuilder: ...
    def setExternalRenderAllowed(self, allowExternal: bool) -> GraphStateBuilder: ...
    def setHotRender(self, isHot: bool) -> GraphStateBuilder: ...
    def setIgnoreTaskCaching(self, ignoreCaching: bool) -> GraphStateBuilder: ...
    def setMaxTimeSamples(self, maxSamples: int) -> GraphStateBuilder: ...
    def setROI(self, roi: list[tuple[float, float, float, float]]) -> GraphStateBuilder: ...
    def setRenderMethodName(self, name: str) -> GraphStateBuilder: ...
    def setSampleRateX(self, sampleRate: float) -> GraphStateBuilder: ...
    def setSampleRateY(self, sampleRate: float) -> GraphStateBuilder: ...
    def setShutterClose(self, time: float) -> GraphStateBuilder: ...
    def setShutterOpen(self, time: float) -> GraphStateBuilder: ...
    @overload
    def setStaticEntry(self, path: str, attr) -> GraphStateBuilder: ...
    @overload
    def setStaticEntry(self, path: str, value: int) -> GraphStateBuilder: ...
    @overload
    def setStaticEntry(self, path: str, value: float) -> GraphStateBuilder: ...
    @overload
    def setStaticEntry(self, path: str, value: str) -> GraphStateBuilder: ...
    def setTime(self, time: float) -> GraphStateBuilder: ...
    def setView(self, view: str) -> GraphStateBuilder: ...

class GroupNode(Node):
    def __init__(self, *args, **kwargs) -> None: ...
    def checkChildrenForCycles(self) -> bool: ...
    def getChild(self, name: str) -> Node: ...
    def getChildByIndex(self, index: int) -> Any: ...
    def getChildren(self) -> list[Node]: ...
    def getNumChildren(self) -> int: ...
    def getReturnPort(self, name: str) -> Port: ...
    def getSendPort(self, name: str) -> Port: ...
    def isContentLocked(self) -> bool: ...
    def setContentLocked(self, value: bool) -> None: ...
    def setLocked(self, value: bool) -> None: ...

class Node:
    def __init__(self, *args, **kwargs) -> None: ...
    def addInputPort(self, name: str) -> NodegraphAPI.Port: ...
    def addInputPortAtIndex(self, name: str, index: int) -> NodegraphAPI.Port: ...
    def addOutputPort(self, name: str) -> NodegraphAPI.Port: ...
    def addOutputPortAtIndex(self, name: str, index: int) -> NodegraphAPI.Port: ...
    def customReset(self) -> bool: ...
    def delete(self) -> bool: ...
    def getAttributes(self) -> dict: ...
    def getBaseType(self) -> str: ...
    def getBoolValue(self, name: str, time: float) -> bool: ...
    def getBypassRouting(self) -> dict: ...
    def getDefaultParameter(self, *args, **kwargs): ...
    def getGraphState(self, *args, **kwargs): ...
    def getInfoString(self) -> str: ...
    def getInputPort(self, *args, **kwargs): ...
    def getInputPortAndGraphState(self, outputPort: NodegraphAPI.Port, graphState: NodegraphAPI.GraphState) -> tuple: ...
    def getInputPortByIndex(self, index: int) -> NodegraphAPI.Port: ...
    def getInputPorts(self) -> list: ...
    def getInputSource(self, name: str, graphState: NodegraphAPI.GraphState) -> tuple: ...
    def getName(self) -> str: ...
    def getNumInputPorts(self) -> int: ...
    def getNumOutputPorts(self) -> int: ...
    def getOutputPort(self, *args, **kwargs): ...
    def getOutputPortByIndex(self, index: int) -> NodegraphAPI.Port: ...
    def getOutputPorts(self) -> list: ...
    def getParameter(self, name: str) -> Parameter: ...
    def getParameterValue(self, name: str, time: float) -> Any: ...
    def getParameters(self) -> GroupParameter: ...
    def getParent(self, *args, **kwargs): ...
    def getSourcePort(self, port: NodegraphAPI.Port, graphState: NodegraphAPI.GraphState) -> tuple: ...
    def getType(self) -> str: ...
    def isAutoRenameAllowed(self) -> bool: ...
    def isBypassed(self) -> bool: ...
    def isLocked(self, considerParents: bool = ...) -> bool: ...
    def isMarkedForDeletion(self) -> bool: ...
    def isRenameAllowed(self) -> bool: ...
    def isResetPossible(self) -> bool: ...
    def loadEnd(self) -> Any: ...
    def parseParameters(self, parameterString: str) -> Any: ...
    def removeInputPort(self, name: str) -> bool: ...
    def removeOutputPort(self, name: str) -> bool: ...
    def renameInputPort(self, *args, **kwargs): ...
    def renameOutputPort(self, *args, **kwargs): ...
    def setAttributes(self, attributes: dict) -> Any: ...
    def setAutoRenameAllowed(self, autoRenameAllowed: bool) -> Any: ...
    def setBypassRouting(self, portRouting: dict) -> Any: ...
    def setBypassed(self, bypassed: bool) -> Any: ...
    def setLocked(self, locked: bool) -> Any: ...
    def setName(self, name: str, updateExpressions: bool = ...) -> str: ...
    def setParent(self, parentNode, notify: bool = ...) -> bool: ...
    def setRenameAllowed(self, renameAllowed: bool) -> Any: ...
    def setType(self, typeName: str) -> Any: ...
    def __hash__(self) -> int: ...

class Parameter:
    INTERPOLATION_CONSTANT: ClassVar[int] = ...
    INTERPOLATION_LINEAR: ClassVar[int] = ...
    REFERENCECOLLECTIONTARGETS_NODES: ClassVar[int] = ...
    REFERENCECOLLECTIONTARGETS_OMIT_DIRECT: ClassVar[int] = ...
    REFERENCECOLLECTIONTARGETS_OMIT_GRAPHSTATE: ClassVar[int] = ...
    REFERENCECOLLECTIONTARGETS_PARAMS: ClassVar[int] = ...
    def __init__(self) -> None: ...
    def buildXmlIO(self, *args, **kwargs): ...
    def createChildGroup(self, name: str, index: int = ...) -> Parameter: ...
    def createChildNumber(self, name: str, value: float, index: int = ...) -> Parameter: ...
    def createChildNumberArray(self, name: str, size: int, index: int = ...) -> Parameter: ...
    def createChildString(self, name: str, value: str, index: int = ...) -> Parameter: ...
    def createChildStringArray(self, name: str, size: int, index: int = ...) -> Parameter: ...
    def createChildXmlIO(self, element, index: int = ...) -> Parameter: ...
    def deleteChild(self, child: Parameter) -> None: ...
    def finalizeValue(self) -> None: ...
    def getAutoKey(self) -> bool: ...
    def getChild(self, name: str) -> Parameter: ...
    def getChildByIndex(self, index: int) -> Parameter: ...
    def getChildren(self) -> list[Parameter]: ...
    def getCurve(self) -> Any: ...
    def getExpression(self) -> str: ...
    def getExpressionError(self) -> str: ...
    def getFileSequenceValue(self, time: float) -> str: ...
    def getFullName(self, includeNodeName: bool = ...) -> str: ...
    def getHintString(self) -> str: ...
    def getIndex(self) -> int: ...
    def getInterpolation(self) -> int: ...
    def getKeys(self) -> list[float]: ...
    def getName(self) -> str: ...
    def getNode(self) -> Node: ...
    def getNumChildren(self) -> int: ...
    def getOpaqueDataInfo(self) -> dict: ...
    def getParent(self) -> Parameter: ...
    def getReferences(self, time: float) -> list[str]: ...
    def getSamples(self) -> Any: ...
    def getTupleSize(self) -> int: ...
    def getType(self) -> str: ...
    def getValue(self, time: float) -> Any: ...
    def getValueXmlIO(self, element, time: float) -> None: ...
    def getXML(self, forcePersistant: bool = ...) -> str: ...
    def hasFloatingCurve(self) -> bool: ...
    def hasKey(self, time: float) -> bool: ...
    def hasUseNodeDefault(self) -> bool: ...
    def insertArrayElement(self, index: int) -> Parameter: ...
    def isAnimated(self) -> bool: ...
    def isExpression(self) -> bool: ...
    def isPersistant(self) -> bool: ...
    def makeConstant(self, time: float) -> None: ...
    @overload
    def moveKeys(self, times: list[float], dt: float) -> None: ...
    @overload
    def moveKeys(self, times: Set[float], dt: float) -> None: ...
    def parseXML(self, xml: str) -> None: ...
    def parseXmlIO(self, element) -> bool: ...
    @classmethod
    def popReferenceCollectionStack(cls) -> tuple: ...
    @classmethod
    def pushReferenceCollectionStack(cls, targets: int = ...) -> None: ...
    def readOpaqueDataFromFile(self, filenames: list[str]) -> None: ...
    def readOpaqueDataFromXmlIO(self, element) -> None: ...
    def removeArrayElement(self, index: int) -> None: ...
    def removeArrayElements(self, index: int, count: int) -> None: ...
    def removeKey(self, time: float) -> None: ...
    @overload
    def removeKeys(self, times: list[float]) -> None: ...
    @overload
    def removeKeys(self, times: Set[float]) -> None: ...
    def renameExpression(self, oldNodeName: str, newNodeName: str) -> None: ...
    def reorderChild(self, child: Parameter, index: int) -> None: ...
    def reorderChildren(self, oldIndex: int, newIndex: int, count: int) -> None: ...
    def reparentAtomic(self, parent: Parameter) -> str: ...
    def reparentExpression(self, oldPath: str, newPath: str) -> None: ...
    def replaceXML(self, xml: str, removeChildren: bool = ...) -> None: ...
    def replaceXmlIO(self, element, removeChildren: bool = ...) -> None: ...
    def resetFloatingCurve(self) -> None: ...
    def resizeArray(self, length: int) -> None: ...
    def sendSignal_finalize(self, **kwargs) -> None: ...
    def sendSignal_setKey(self, keyTime: float) -> None: ...
    def sendSignal_setValue(self, **kwargs) -> None: ...
    def setAutoKey(self, value: bool) -> None: ...
    def setCurve(self, curve, final: bool = ...) -> None: ...
    def setExpression(self, expression: str, enable: bool = ...) -> None: ...
    def setExpressionFlag(self, value: bool) -> None: ...
    def setHintString(self, hint: str) -> None: ...
    def setInterpolation(self, interpolation: int, final: bool = ...) -> None: ...
    def setKey(self, time: float, sendNotifyMessage: bool = ...) -> None: ...
    def setName(self, newName: str) -> str: ...
    def setPersistant(self, arg0: bool) -> None: ...
    def setSamples(self, samples: dict[float, list[float]], final: bool = ...) -> None: ...
    def setTupleSize(self, size: int) -> None: ...
    def setUseNodeDefault(self, value: bool) -> None: ...
    def setValue(self, value: object, time: float, final: bool = ..., sendNotifyMessage: bool = ...) -> None: ...
    def setValueAutoKey(self, value: object, time: float, final: bool = ...) -> None: ...
    def setValueXmlIO(self, element, time: float, final: bool = ..., sendNotifyMessage: bool = ...) -> None: ...
    def useNodeDefault(self) -> bool: ...
    def writeOpaqueDataToFile(self, filenames: list[str]) -> None: ...
    def writeOpaqueDataToXmlIO(self, *args, **kwargs): ...
    def __hash__(self) -> int: ...

class Port:
    FLAVOR_DEPENDENCY: ClassVar[int] = ...
    FLAVOR_MASK: ClassVar[int] = ...
    FLAVOR_STANDARD: ClassVar[int] = ...
    TYPE_CONSUMER: ClassVar[int] = ...
    TYPE_PRODUCER: ClassVar[int] = ...
    def __init__(self) -> None: ...
    def addOrUpdateMetadata(self, key: str, value: str) -> None: ...
    def connect(self, other: Port, doCycleCheck: bool = ...) -> bool: ...
    def deleteMetadata(self, key: str) -> None: ...
    def disconnect(self, other: Port) -> bool: ...
    def getAttributes(self) -> dict: ...
    def getColor(self) -> tuple: ...
    def getConnectedPort(self, index: int) -> Port: ...
    def getConnectedPorts(self) -> list[Port]: ...
    def getDisplayName(self, prefixWithPageNames: bool = ...) -> str: ...
    def getFlavor(self) -> int: ...
    def getIndex(self) -> int: ...
    def getMetadata(self, key: str) -> Any: ...
    def getMetadataDict(self) -> dict[str, str]: ...
    def getName(self) -> str: ...
    def getNode(self) -> Node: ...
    def getNumConnectedPorts(self) -> int: ...
    def getTags(self) -> list: ...
    def getType(self) -> int: ...
    def hasCustomColor(self) -> bool: ...
    def isConnected(self, other: Port) -> bool: ...
    def setAttributes(self, attrs: dict) -> None: ...
    def setColor(self, r: float, g: float, b: float, isCustom: bool = ...) -> None: ...
    def setTags(self, tags: list[str]) -> None: ...
    def __hash__(self) -> int: ...

class PythonGroupNode(GroupNode):
    def __init__(self) -> None: ...

class PythonNode(Node):
    def __init__(self) -> None: ...
    @overload
    def getInputPortAndGraphState(self, port: Port, graphState: GraphState) -> tuple: ...
    @overload
    def getInputPortAndGraphState(self, port: Port, time: float) -> tuple: ...

def BuildAttrFromGroupParameter(param: Parameter, graphState: GraphState = ..., multisampleDefault: bool = ..., numberAttrTypeDefault: object = ..., includeEmptyGroups: bool = ..., groupInherit: bool = ...) -> Any: ...
def BuildAttrFromStringParameter(param: Parameter, graphState: GraphState = ..., multisample: bool = ...) -> Any: ...
def BuildDoubleAttrFromNumberParameter(param: Parameter, graphState: GraphState = ..., multisample: bool = ...) -> Any: ...
def BuildFloatAttrFromNumberParameter(param: Parameter, graphState: GraphState = ..., multisample: bool = ...) -> Any: ...
def BuildIntAttrFromNumberParameter(param: Parameter, graphState: GraphState = ..., multisample: bool = ...) -> Any: ...
def GetGraphStateAtPort(searchPort: Port, startPort: Port, startGraphState: GraphState) -> Any: ...
def GetLocalGraphState() -> Any: ...
def GetSampleTimes(shutterOpen: float, shutterClose: float, maxTimeSamples: int, useSinglePrecision: bool = ...) -> tuple: ...
def GetSampleTimesFromGraphState(graphState: GraphState, useSinglePrecision: bool = ...) -> tuple: ...
def GetSingleSampleTimeFromGraphState(graphState: GraphState) -> float: ...
def IsParameterConstant(parameter: Parameter) -> bool: ...
def PopLocalGraphState() -> None: ...
def PushLocalGraphState(graphState: GraphState) -> None: ...
def curve_writeCurveXML(curve) -> str: ...
def node_addNodeFlavor(nodeType: str, flavor: str) -> None: ...
def node_clearFlavorNodes(flavor: str) -> None: ...
def node_createNode(nodeType: str, parent: Node = ...) -> Any: ...
def node_getAllConnectedInputs(sourceNodes: typing.Iterable) -> set: ...
def node_getAllFlavors() -> list[str]: ...
def node_getAllNodes(includeDeleted: bool = ..., sortByName: bool = ...) -> list: ...
def node_getAllNodesByType(nodeType: str, includeDeleted: bool = ..., sortByName: bool = ...) -> list: ...
def node_getCurrentGraphState() -> GraphState: ...
def node_getCurrentTime() -> float: ...
def node_getFlavorNodes(flavor: str, existsFlag: bool) -> list[str]: ...
def node_getGraphState(time: float) -> GraphState: ...
def node_getNode(nodeName: str) -> Any: ...
def node_getNodeFlavors(nodeType: str) -> list[str]: ...
def node_getOriginalProjectAssetID() -> str: ...
def node_getProjectAssetID() -> str: ...
def node_getTypeList() -> str: ...
def node_isLoading() -> bool: ...
def node_needsRedraw() -> bool: ...
def node_registerExtension(extension: object) -> None: ...
def node_registerPythonGroupType(typeName: str, constructor: typing.Callable) -> None: ...
def node_registerPythonNodeFactory(nodeType: str, factory: typing.Callable) -> None: ...
def node_registerPythonNodeType(typeName: str, constructor: typing.Callable) -> None: ...
def node_registerQueryFlag(callbacK: typing.Callable) -> None: ...
def node_removeNodeFlavor(nodeType: str, flavor: str) -> None: ...
def node_setExpressionObject(arg0: object) -> None: ...
def node_setIsLoading(value: bool) -> None: ...
def node_setNeedsRedraw(arg0: bool) -> None: ...
def node_setOriginalProjectAssetID(assetId: str) -> None: ...
def node_setProjectAssetID(assetId: str) -> None: ...
def node_setRootNode(rootNode: Node) -> None: ...
def notify_areNotificationsEnabled() -> bool: ...
def notify_setNotificationFn(handler: typing.Callable) -> None: ...
def notify_setNotificationsEnabled(arg0: bool) -> None: ...
def parameter_getAllExpressionedParameters(errorsOnly: bool = ..., noErrorsOnly: bool = ...) -> Set[Parameter]: ...
def parameter_getAutoKeyAll() -> bool: ...
def parameter_getExpressionReferences(referents: object = ..., referers: object = ...) -> dict: ...
def parameter_popExpressionCache() -> None: ...
def parameter_pushExpressionCache() -> None: ...
def parameter_resetAllFloatingCurves() -> None: ...
def parameter_setAutoKeyAll(autoKeyAll: bool) -> None: ...
def parameter_setExpressionGlobals(globals: object) -> None: ...
def parameter_setExpressionLocalFn(locals: typing.Callable) -> None: ...
def parameter_setOpaqueParameterBuildXmlIOHandler(handler: object) -> None: ...
def port_registerComplexConnector(connector: typing.Callable) -> None: ...
def pyerror_occurred() -> bool: ...
def registerBuiltinFactories() -> None: ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import Utils.EventModule as EventModule
import NodegraphAPI as NodegraphAPI
import Utils as Utils
import collections
from PyUtilModule.WorkingSet import WorkingSet as WorkingSet
from PyUtilModule.WorkingSetManager import WorkingSetManager as WorkingSetManager
from Utils.Decorators import deprecated as deprecated
from _typeshed import Incomplete
from typing import Set, Tuple

ExpandedWorkingSetName: str
PinnedWorkingSetName: str
SelectedWorkingSetName: str
WorkingSetNames: tuple

class Scenegraph:
    def __init__(self) -> None: ...
    def _Scenegraph__emitSelectionChanged(self, selectedLocations, deselectedLocations, sender): ...
    def _Scenegraph__expandedLocationStateChangedCallback(self, locationStateChanges, workingSet, sender): ...
    def _Scenegraph__onSceneLoadOrNewScene(self, **kwargs): ...
    def _Scenegraph__pinnedLocationStateChangedCallback(self, locationStateChanges, workingSet, sender): ...
    def _Scenegraph__selectedLocationStateChangedCallback(self, locationStateChanges, workingSet, sender): ...
    def _Scenegraph__setSelectedLocationStateChangedCallbackEnabled(self, enabled): ...
    def _Scenegraph__validateLocationArgument(self, functionName: str, location: str): ...
    def _Scenegraph__validateLocationsArgument(self, functionName: str, locations: list | tuple | set[str] | str): ...
    def addOpenLocations(self, locations, replace, sender: Incomplete | None = ...): ...
    def addPinnedLocation(self, location, sender: Incomplete | None = ...): ...
    def addPinnedLocations(self, locations, replace, sender: Incomplete | None = ...): ...
    def addSelectedLocations(self, locations, replace, history: bool = ..., sender: Incomplete | None = ...): ...
    def clearBookmarkSets(self, sender: Incomplete | None = ...): ...
    def clearOpenLocations(self, sender: Incomplete | None = ...): ...
    def clearPinnedLocations(self, sender: Incomplete | None = ...): ...
    def clearSelection(self, sender: Incomplete | None = ...): ...
    def closeLocations(self, locationPathOrPaths, recursive: bool = ..., sender: Incomplete | None = ...): ...
    def ensureLocationVisible(self, location, sender: Incomplete | None = ...): ...
    def getBookmarkSets(self): ...
    def getImplicitResolversActive(self): ...
    def getOpenLocations(self): ...
    def getPinnedLocations(self): ...
    def getSelectedLocations(self): ...
    def getVisibleSelectedLocations(self): ...
    def graphId(self): ...
    def invertSelectedLocations(self, locations, history: bool = ..., sender: Incomplete | None = ...): ...
    def isLocationOpen(self, location): ...
    def isLocationPinned(self, location): ...
    def isLocationSelected(self, location: str, checkAncestors: bool = ...) -> bool: ...
    def isLocationVisible(self, location): ...
    def isViewerVisibilityFollowingWorkingSet(self) -> bool: ...
    def locationHasPinnedDescendants(self, location): ...
    def openLocations(self, locationPathOrPaths, sender: Incomplete | None = ...): ...
    def openLocationsRecursive(self, locationPathOrPaths, sender: Incomplete | None = ...): ...
    def pushSelectionHistory(self): ...
    def removePinnedLocation(self, location, sender: Incomplete | None = ...): ...
    def removePinnedLocations(self, locations, sender: Incomplete | None = ...): ...
    def removeSelectedLocations(self, locations, history: bool = ..., sender: Incomplete | None = ...): ...
    def selectionHistoryBackward(self): ...
    def selectionHistoryCanGoBackward(self): ...
    def selectionHistoryCanGoForward(self): ...
    def selectionHistoryForward(self): ...
    def setBookmarkSets(self, openSet, selectionSet, pinSet): ...
    def setImplicitResolversActive(self, active): ...
    def setLocationOpen(self, location, isOpen, sender: Incomplete | None = ...): ...
    def setLocationSelected(self, location, selected: bool = ..., sender: Incomplete | None = ...): ...
    def setViewerVisibilityFollowingWorkingSet(self, mode: bool): ...

class SelectionHistory:
    def __init__(self) -> None: ...
    def backward(self): ...
    def canGoBackward(self): ...
    def canGoForward(self): ...
    def clear(self): ...
    def commit(self): ...
    def forward(self): ...
    def getCurrent(self): ...
    def getHistory(self): ...
    def update(self, items): ...

class _OrderedSet(collections.OrderedDict):
    def append(self, item): ...
    def discard(self, item): ...
    def insert(self, item): ...

def getActiveScenegraph(): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import PyFnGeolibServices.ExpressionMath as ExpressionMath
import PyFnAttribute as FnAttribute
import PyFnGeolibServices as FnGeolibServices
import PackageSuperToolAPI.NodeUtils as NU
import NodegraphAPI as NodegraphAPI
import Nodes3DAPI as Nodes3DAPI
import PackageSuperToolAPI.Packages
import PackageSuperToolAPI.Packages as Packages
import PyFnAttribute
from NodegraphAPI.SuperTool import SuperTool
from PyUtilModule.VirtualKatana import Decorators as Decorators
from _typeshed import Incomplete
from typing import ClassVar, Set, Tuple

API_VERSION: int
SYNC_SELECTION_IN_OUT: float
SYNC_SELECTION_OFF: float
SYNC_SELECTION_OUT: float

class BaseNode(SuperTool):
    VERSION: ClassVar[int] = ...
    def __init__(self, version: Incomplete | None = ...) -> None: ...
    @classmethod
    def _BaseNode__getEditPackageForLocationPath(cls, mainNode: NodegraphAPI.Node, locationPath: str) -> PackageSuperToolAPI.Packages.Package | None: ...
    def _BaseNode__getNotInInputMarkingOpScript(self): ...
    def _BaseNode__getNotInInputStrippingOpScript(self): ...
    def _BaseNode__getOpScriptForNonAdoptedStripping(self): ...
    def adoptLocationForEditing(self, locationPath: str, locationAttributes: Incomplete | None = ...): ...
    def adoptLocationsForEditing(self, locationPaths: list[str], locationAttributes: Incomplete | None = ...): ...
    def canAdoptLocationForEditing(self, location: str) -> bool: ...
    def canAdoptLocationsForEditing(self, locations: list[str]) -> bool: ...
    def getAPIVersion(self) -> int: ...
    @classmethod
    def getDefaultRootLocation(cls) -> str: ...
    @classmethod
    def getItemListAttributeName(cls) -> str | None: ...
    def getLocationAttributes(self, locationPath: str) -> PyFnAttribute.GroupAttribute | None: ...
    def getPackageForPath(self, locationPath: str, raiseOnMissing: bool = ..., locationAttributes: Incomplete | None = ..., includeEditPackages: bool = ..., createDummyOnMissing: bool = ...) -> PackageSuperToolAPI.Packages.Package | None: ...
    def getRegisteredDisplayPackageClasses(self) -> list[PackageSuperToolAPI.Packages.Package]: ...
    def getRegisteredPackageClasses(self) -> list[tuple[str, type]]: ...
    def getRootLocation(self) -> str: ...
    def getRootPackage(self) -> PackageSuperToolAPI.Packages.Package: ...
    def getShowIncomingScene(self) -> bool: ...
    @classmethod
    def getSuperToolName(cls) -> str: ...
    def getSyncSelection(self) -> int: ...
    def getVersion(self) -> int: ...
    def isLocationAdopted(self, locationPath: str) -> bool: ...
    def setRootLocation(self, locationPath: str): ...
    def setShowIncomingScene(self, state: bool): ...
    def setSyncSelection(self, syncSelection: int): ...
    def setVersion(self, version: int): ...
    @classmethod
    def superToolRegistered(cls): ...
    def upgrade(self, rootPackage: RootPackage): ...

def RegisterTagAdoptedNodeType(): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import UI4.App as App
import KatanaResources as KatanaResources
import UI4.MainUI as MainUI
import PyQt5.QtCore
import PyQt5.QtGui
import PyQt5.QtWidgets
import PyQt5.QtCore as QtCore
import PyQt5.QtGui as QtGui
import PyQt5.QtWidgets as QtWidgets
import UI4.Widgets.BaseWidgets
import Utils as Utils
import typing
from UI4.Widgets.BaseWidgets import BaseFrame as BaseFrame
from UI4.Widgets.IconLabelFrame import IconLabelFrame as IconLabelFrame
from _typeshed import Incomplete
from typing import ClassVar, Set, Tuple

class BaseTab(UI4.Widgets.BaseWidgets.BaseFrame):
    tabClosed: ClassVar[PyQt5.QtCore.pyqtSignal] = ...
    def __init__(self, parent: PyQt5.QtWidgets.QWidget | None, flags: int = ..., deprecatedBy: Incomplete | None = ...) -> None: ...
    def _BaseTab__addDeprecationWarningFrame(self): ...
    def _BaseTab__linkClickCallback(self, link: str): ...
    def _BaseTab__on_pref_changed(self, eventType: str | None, eventID: typing.Hashable, prefKey: str, prefValue: object): ...
    def _BaseTab__removeDeprecationWarningFrame(self): ...
    def _BaseTab__showDeferredWidgets(self, events): ...
    def _showWidgetLater(self, widget: PyQt5.QtWidgets.QWidget): ...
    def applySettings(self, settings: dict): ...
    def closeEvent(self, event: PyQt5.QtGui.QCloseEvent): ...
    def getMenuBar(self) -> PyQt5.QtWidgets.QMenuBar | None: ...
    def getSettings(self) -> dict: ...
    def getTabTitle(self) -> str | None: ...
    def on_animation_finished(self): ...
    def on_dismissButton_clicked(self): ...
    def on_replaceTabButton_clicked(self): ...
    def showEvent(self, event: PyQt5.QtGui.QShowEvent): ...
    def __lt__(self, other) -> bool: ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import PyQt5.QtCore
import PyQt5.QtCore as QtCore
from typing import ClassVar, Set, Tuple

ErrorMessageUpdate: int
FrameBufferRectUpdate: int
RenderCancelledUpdate: int
RenderCompletedUpdate: int
RenderProgressUpdate: int
RenderUpdate: int
TileRenderOrderUpdate: int

class CatalogEvent(PyQt5.QtCore.QEvent):
    QtEventType: ClassVar[int] = ...
    def __init__(self, catalogEventType, sequenceID) -> None: ...
    def getCatalogEventType(self): ...
    def getSequenceID(self): ...
    def type(self): ...

class CatalogEventReceiver(PyQt5.QtCore.QObject):
    def __init__(self, eventHandler) -> None: ...
    def customEvent(self, event): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import ConditionalStateGrammar as ConditionalStateGrammar
import ConfigurationAPI_cmodule as Configuration
import Nodes3DAPI.DynamicParameterUtil as DynamicParameterUtil
import PyFnAttribute as FnAttribute
import PyFnGeolib as FnGeolib
import Nodes3DAPI.Node3D as Node3D
import NodegraphAPI as NodegraphAPI
import Nodes3DAPI_cmodule as Nodes3DAPI_cmodule
import PyXmlIO as PyXmlIO
import Utils as Utils
import PyXmlIO as XmlIO
from ConditionalStateGrammar.Parser import ParseCSG as ParseCSG
from Nodes3DAPI.GenericAssign import GenericAssign as GenericAssign
from _typeshed import Incomplete
from typing import ClassVar, Set, Tuple

_ExtraHints: dict
_Parameter_XML: str
_baseXML: None

class RenderOutputDefine(GenericAssign):
    _RenderOutputDefine__incomingCookKey: ClassVar[str] = ...
    __pychecker__: ClassVar[str] = ...
    def __init__(self) -> None: ...
    def _getIncomingSceneOpAndLocation(self, port: NodegraphAPI.Port, graphState: NodegraphAPI.GraphState, transaction): ...
    def _getOpChain(self, interface): ...
    def _updateParameters(self, groupAttr, force: bool = ..., defaultAttr: Incomplete | None = ...): ...
    def addParameterHints(self, attrName, inputDict): ...
    def checkDynamicParameters(self, *args, **kwds): ...
    def updateParameters(self, universalAttr: Incomplete | None = ..., buildParameters: bool = ...): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import NodegraphAPI as NodegraphAPI
import NodegraphAPI_cmodule
from _typeshed import Incomplete
from typing import Set, Tuple

_DefaultShapeAttrs: tuple
_ExtraHints: dict
_Parameter_XML: str

class GroupMergeNode(NodegraphAPI_cmodule.PythonGroupNode):
    def __init__(self) -> None: ...
    def _GroupMergeNode__buildMergeNode(self): ...
    def _GroupMergeNode__getMergeNode(self): ...
    def addParameterHints(self, attrName, inputDict): ...
    def allowChildReparentingViaDrag(self, childNode): ...
    def buildChildNode(self, adoptNode: Incomplete | None = ...): ...
    def canAdoptNodes(self, nodes): ...
    def deleteChildNode(self, childNode): ...
    def getChildNodeType(self): ...
    def getChildNodes(self): ...
    def getDefaultNodeShapeAttrs(self): ...
    def getDisplayNameExpression(self): ...
    def getDisplayNameForChildNode(self, childNode): ...
    def getInfoString(self): ...
    def positionChildNodes(self): ...
    def preprocessChildReparentingViaDrag(self, childNode): ...
    def reorderChildNode(self, fromIndex, toIndex): ...
    def setChildNodeType(self, nodeType): ...
    def setDisplayNameExpression(self, exprText): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import NodegraphAPI as NodegraphAPI
import Nodes3DAPI as Nodes3DAPI
import PyXmlIO as PyXmlIO
import QT4FormWidgets as QT4FormWidgets
import PyQt5.QtCore as QtCore
import PyQt5.QtGui as QtGui
import PyQt5.QtWidgets as QtWidgets
import UI4.Widgets.SortablePanel as SortablePanel
import UI4 as UI4
import UI4.Widgets.SortablePanel
from QT4FormWidgets.StringFormWidget import StringFormWidget
from typing import Set, Tuple

class NodeNameArrayEditor(UI4.Widgets.SortablePanel.SortablePanelFormWidget):
    def __init__(self, parent, valuePolicy, widgetFactory) -> None: ...
    def _NodeNameArrayEditor__addNodeChildren(self, replace: bool = ...): ...
    def _NodeNameArrayEditor__appendEntry(self): ...
    def _NodeNameArrayEditor__appendNodegraphSelection(self): ...
    def _NodeNameArrayEditor__findAllNodes(self): ...
    def _NodeNameArrayEditor__replaceNodegraphSelection(self): ...
    def buildAddMenu(self, menu): ...
    def panelDeleted(self, index): ...
    def panelReordered(self, oldPos, newPos): ...
    def updatePanels(self): ...

class NodeNameFormWidget(StringFormWidget):
    def __init__(self, parent, policy, factory) -> None: ...
    def _NodeNameFormWidget__fillMenu(self, menu): ...
    def _NodeNameFormWidget__getNodeFromEvent(self, event): ...
    def _NodeNameFormWidget__setFromText(self, text): ...
    def _buildControlWidget(self, layout): ...
    def _checkControlWidget(self): ...
    def _filterNodes(self, unfilteredNodes): ...
    def _updateControlWidget(self): ...
    def dragEnterEvent(self, event): ...
    def dropEvent(self, event): ...
    def getEditWidget(self): ...
    def keyPressEvent(self, event): ...
    def setReadOnly(self, value): ...

class NodeNameIndexFormWidget(NodeNameFormWidget):
    def __init__(self, parent, parentPolicy, index, factory) -> None: ...
    def setPolicyIndex(self, index): ...

class NodeNamePanel(UI4.Widgets.SortablePanel.SortablePanelBase):
    def __init__(self, parent, policy, index, widgetFactory) -> None: ...
    def setPolicyIndex(self, index): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import PyFnAttribute
from typing import Set, Tuple

TOP_LEVEL_CONTAINER_NAME: str

def FlushCaches() -> None: ...
def GetProceduralArgs(assetId: str) -> PyFnAttribute.Attribute: ...
def ParseArgsFile(fileName: str) -> PyFnAttribute.Attribute: ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import Callbacks as Callbacks
import ConfigurationAPI_cmodule as Configuration
import Utils.EventModule as EventModule
import PyFnAttribute as FnAttribute
import PyFnGeolib as FnGeolib
import PyFnGeolibProducers as FnGeolibProducers
import Nodes3DAPI.Node3DEventTypes as Node3DEventTypes
import Nodes3DAPI.Node3D_geolib3 as Node3D_geolib3
import NodegraphAPI as NodegraphAPI
import NodegraphAPI_cmodule
import Nodes2DAPI as Nodes2DAPI
import Nodes3DAPI.Node3D_geolib3
import PyFnAttribute
import PyFnGeolib
import PyFnGeolibProducers
import RenderingAPI as RenderingAPI
import Utils as Utils
import typing
from Nodes3DAPI.IncomingSceneOpDelegates import IncomingSceneOpDelegate as IncomingSceneOpDelegate, OutgoingAttributesDelegate as OutgoingAttributesDelegate, RegisterIncomingSceneOpDelegate as RegisterIncomingSceneOpDelegate, RegisterOutgoingAttributesDelegate as RegisterOutgoingAttributesDelegate
from Nodes3DAPI.Node3D_geolib3 import ActivatePort as ActivatePort, CommitChanges as CommitChanges, DeactivatePort as DeactivatePort, GetDefaultSourceOp as GetDefaultSourceOp, GetRuntime as GetRuntime, IsProcessing as IsProcessing, ManualUpdate as ManualUpdate, MarkPortOpClientDirty as MarkPortOpClientDirty, RegisterCommitIdCallback as RegisterCommitIdCallback, RegisterPortOpClient as RegisterPortOpClient, UnregisterPortOpClient as UnregisterPortOpClient, UpdatePortOpClients as UpdatePortOpClients
from Nodes3DAPI.NodeTypeBuilder import NodeTypeBuilder as NodeTypeBuilder
from Nodes3DAPI.OpCacheManager import OpCacheManager as OpCacheManager, g_opCacheManager as g_opCacheManager
from Nodes3DAPI.ScenegraphMask import GetScenegraphMaskEnabled as GetScenegraphMaskEnabled, GetScenegraphMaskLocationsAndRoot as GetScenegraphMaskLocationsAndRoot, GetVisibilityFollowsWorkingSet as GetVisibilityFollowsWorkingSet
from Nodes3DAPI.TerminalOpDelegates.TerminalOpDelegate import TerminalOpDelegate
from PyUtilModule.WorkingSetManager import WorkingSetManager as WorkingSetManager
from _typeshed import Incomplete
from typing import ClassVar, Set, Tuple

class ImplicitResolverStage:
    AfterLookFileResolvePostprocessResolver: ClassVar[int] = ...
    AfterPostprocessResolvers: ClassVar[int] = ...
    AfterStandardResolvers: ClassVar[int] = ...
    AfterViewerResolvers: ClassVar[int] = ...
    BeforeLookFileResolvePostprocessResolver: ClassVar[int] = ...
    BeforePreprocessResolvers: ClassVar[int] = ...
    BeforeStandardResolvers: ClassVar[int] = ...
    BeforeViewerResolvers: ClassVar[int] = ...

class Node3D(NodegraphAPI_cmodule.PythonNode, Nodes3DAPI.Node3D_geolib3.NodeGeolib3):
    def __init__(self) -> None: ...
    def _getInvalidConnectionsErrorMessage(self, errorMessages): ...
    def getAll3DInputs(self, graphState: NodegraphAPI.GraphState): ...
    def getPortsWithInvalidConnections(self, errorMessages: Incomplete | None = ...): ...
    def require3DInput(self, portName, graphState: NodegraphAPI.GraphState): ...
    def validateConnection(self, otherOutPort, myInPort, errorMessages: Incomplete | None = ...): ...

def ApplyImplicitResolverOps(txn, op, node: NodegraphAPI.Node, graphState: typing.Optional[NodegraphAPI.GraphState] = ..., extraOpArgsByOpType: Incomplete | None = ...): ...
def ApplyOp(txn, inputOp, opType, opArgs: Incomplete | None = ...): ...
def ApplyRenderSettingsToGraphState(renderSettingsAttr: PyFnAttribute.GroupAttribute, graphState: NodegraphAPI.GraphState, renderUUID: Incomplete | None = ...) -> NodegraphAPI.GraphState: ...
def CreateClient(node: NodegraphAPI.Node, graphState: typing.Optional[NodegraphAPI.GraphState] = ..., portIndex: int = ...): ...
def Get3DPortFromNode(node: NodegraphAPI.Node, graphState: typing.Optional[NodegraphAPI.GraphState] = ..., portIndex: int = ...): ...
def Get3DSourceFromNodeInput(node: NodegraphAPI.Node, inputPortName: str, graphState: NodegraphAPI.GraphState) -> NodegraphAPI.Port | None: ...
def GetExtraParameterValueSourceNodePorts(node: NodegraphAPI.Node) -> list[NodegraphAPI.Port]: ...
def GetExtraParameterValueSourceNodes(node: NodegraphAPI.Node) -> list[NodegraphAPI.Node]: ...
def GetGeometryProducer(node: typing.Optional[NodegraphAPI.Node] = ..., graphState: typing.Optional[NodegraphAPI.GraphState] = ..., portIndex: int = ...) -> PyFnGeolibProducers.GeometryProducer | None: ...
def GetOp(txn, node: NodegraphAPI.Node, graphState: typing.Optional[NodegraphAPI.GraphState] = ..., portIndex: int = ..., applyTerminalOpDelegates: bool = ...): ...
def GetOpChain(txn, node: NodegraphAPI.Node, graphState: typing.Optional[NodegraphAPI.GraphState] = ..., portIndex: int = ...): ...
def GetRegisteredImplicitResolvers(stage): ...
def GetRegisteredTerminalOpDelegate(name: str) -> TerminalOpDelegate: ...
def GetRegisteredTerminalOpDelegateNames() -> list[str]: ...
def GetRenderOp(txn, node: NodegraphAPI.Node, graphState: typing.Optional[NodegraphAPI.GraphState] = ..., portIndex: int = ..., useMaxSamples: bool = ..., applyRenderSelectedMask: bool = ..., update3DNodes: bool = ..., visitedState: Incomplete | None = ..., runtime: Incomplete | None = ..., renderUUID: Incomplete | None = ...): ...
def GetRenderProducer(node: typing.Optional[NodegraphAPI.Node] = ..., graphState: typing.Optional[NodegraphAPI.GraphState] = ..., useMaxSamples: bool = ..., portIndex: int = ..., applyRenderSelectedMask: bool = ...) -> PyFnGeolibProducers.GeometryProducer | None: ...
def GetRenderTerminalOpSpecs(rendererName: str, renderMethodType: str, graphState: NodegraphAPI.GraphState) -> list[tuple[str, PyFnAttribute.GroupAttribute]]: ...
def GetRenderTerminalOps(txn: PyFnGeolib.GeolibRuntimeTransaction, rendererName: str, renderMethodType: str, graphState: typing.Optional[NodegraphAPI.GraphState] = ...) -> list[PyFnGeolib.GeolibRuntimeOp]: ...
def GetRenderThreads(): ...
def RegisterImplicitResolver(stage, opType, opArgs, addSystemArgs: bool = ...): ...
def RegisterTerminalOpDelegate(name: str, terminalOpDelegate: TerminalOpDelegate): ...
def SetExtraParameterValueSourceNodes(node: NodegraphAPI.Node, sourceNodes: list[NodegraphAPI.Node]): ...
def SetRenderThreads(numThreads): ...
def UnregisterTerminalOpDelegate(name: str): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import PyQt5.QtCore
import PyQt5.QtWidgets
import PyQt5.QtWidgets as QtWidgets
from _typeshed import Incomplete
from typing import Set, Tuple

class ManagedAction(PyQt5.QtWidgets.QAction):
    def __init__(self, actionID: str, text: str, parent: PyQt5.QtCore.QObject, icon: Incomplete | None = ...) -> None: ...
    def _ManagedAction__on_changed(self): ...
    def getActionID(self) -> str: ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import QT4FormWidgets as QT4FormWidgets
import PyQt5.QtCore as QtCore
import PyQt5.QtWidgets as QtWidgets
from QT4FormWidgets.FormWidget import FormWidget
from UI4.Widgets.HorizontalDivider import HorizontalDivider as HorizontalDivider
from typing import Set, Tuple

class SeparatorFormWidget(FormWidget):
    def __init__(self, parent, policy, factory) -> None: ...
    def _buildControlWidget(self, layout): ...
    def _buildLabel(self, text): ...
    def _destroyLabel(self): ...
    def setHeight(self, height): ...
    def setSeparatorLabel(self, text): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

from . import Flipbook as Flipbook, Layers as Layers, NodeBreadcrumbsFrame as NodeBreadcrumbsFrame, NodeFindPopupButton as NodeFindPopupButton, NodeGoRootButton as NodeGoRootButton, NodeGoUpButton as NodeGoUpButton, NodeStateFilterFrame as NodeStateFilterFrame, NodegraphWidget as NodegraphWidget
from UI4.Tabs.NodeGraphTab.Flipbook import FlipbookSettingsDialog as FlipbookSettingsDialog
from UI4.Tabs.NodeGraphTab.NodegraphPanel import NodegraphPanel as NodegraphPanel
from typing import Set, Tuple

PluginRegistry: list



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import QT4FormWidgets as FW
import PyFnAttribute as FnAttribute
import PyUtilModule.Hints as Hints
import Naming as Naming
import PyFnAttribute
import PyFnGeolib
import PyQt5.QtCore
import PyQt5.QtWidgets
import QT4FormWidgets.ValuePolicy
import PyQt5.QtCore as QtCore
import PyQt5.QtGui as QtGui
import PyQt5.QtWidgets as QtWidgets
import UI4 as UI4
import Utils as Utils
from UI4.Widgets.AttributeHistoryWidgets import AddAttributeHistoryActionsToMenu as AddAttributeHistoryActionsToMenu
from _typeshed import Incomplete
from typing import ClassVar, Set, Tuple

class AttrStates:
    DEFAULT: ClassVar[int] = ...
    DEFAULT_REFERENCE: ClassVar[int] = ...
    GLOBAL: ClassVar[int] = ...
    GLOBAL_DEFAULT_REFERENCE: ClassVar[int] = ...
    LOCAL: ClassVar[int] = ...

class AttributePolicy(QT4FormWidgets.ValuePolicy.AbstractValuePolicy):
    _AttributePolicy__appearances: ClassVar[dict] = ...
    _AttributePolicy__defaultReferenceAppearance: ClassVar[tuple] = ...
    _AttributePolicy__defaultState: ClassVar[set] = ...
    _AttributePolicy__globalState: ClassVar[set] = ...
    _AttributePolicy__localState: ClassVar[set] = ...
    _AttributePolicy__stateLabels: ClassVar[dict] = ...
    def __init__(self, parent, name, attr, index: Incomplete | None = ..., metadataDict: Incomplete | None = ..., pageEntry: Incomplete | None = ..., locationPath: Incomplete | None = ..., sourceOp: Incomplete | None = ...) -> None: ...
    def _AttributePolicy__getType(self, attr, index: Incomplete | None = ...): ...
    def _AttributePolicy__getWidgetHints(self, metadataDict): ...
    def _appendToStateMenu(self, menu: PyQt5.QtWidgets.QMenu) -> bool: ...
    def _buildChildren(self, attr, metadataDict, pageEntry): ...
    def _getAttr(self): ...
    def _getChildren(self): ...
    def _isPage(self): ...
    def _replaceAttr(self, attr, name: Incomplete | None = ..., metadataDict: Incomplete | None = ..., locationPath: Incomplete | None = ...): ...
    def getArrayChild(self, index): ...
    def getArraySize(self): ...
    def getAttrName(self): ...
    def getAttribute(self) -> PyFnAttribute.Attribute: ...
    def getAttributeTypeName(self) -> str: ...
    def getChildByName(self, name): ...
    def getChildren(self): ...
    def getCoPolicyPathDict(self): ...
    def getMimeData(self) -> PyQt5.QtCore.QMimeData: ...
    def getName(self): ...
    def getNumChildren(self): ...
    def getSceneGraphLocationPath(self) -> str | None: ...
    def getSourceOp(self) -> PyFnGeolib.GeolibRuntimeOp | None: ...
    def getStateAppearance(self, state): ...
    def getStateToolTip(self, state): ...
    def getTooltip(self): ...
    def getType(self): ...
    def getValue(self): ...
    def getValueState(self): ...
    def getWidgetHints(self): ...
    def isLocked(self): ...
    def setSceneGraphLocationPath(self, locationPath: str | None): ...
    def setSourceOp(self, sourceOp: PyFnGeolib.GeolibRuntimeOp | None): ...
    def shouldDisplayState(self): ...

def GetPolicyFromLocationAttrs(locationAttrs, attrPath: Incomplete | None = ...): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import NodegraphAPI
import NodegraphAPI_cmodule as NodegraphAPI_cmodule
import PyXmlIO as PyXmlIO
import Utils as Utils
import NodegraphAPI.Xio as Xio
from typing import Set, Tuple

class UserParameterError(RuntimeError): ...

def CreateButton(node: NodegraphAPI.Node, name, text: str = ..., script: str = ...): ...
def CreateColor(node: NodegraphAPI.Node, name): ...
def CreateGroup(node: NodegraphAPI.Node, name): ...
def CreateNumber(node: NodegraphAPI.Node, name): ...
def CreateNumberArray(node: NodegraphAPI.Node, name): ...
def CreateString(node: NodegraphAPI.Node, name): ...
def CreateStringArray(node: NodegraphAPI.Node, name): ...
def Delete(node: NodegraphAPI.Node, name): ...
def DeleteLiveUserParameter(node: NodegraphAPI.Node): ...
def DeleteUserParameter(node: NodegraphAPI.Node): ...
def ExecuteButton(node: NodegraphAPI.Node, name): ...
def GetLiveUserParameter(node: NodegraphAPI.Node, createIfMissing: bool = ...): ...
def GetUserParameter(node: NodegraphAPI.Node): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import KatanaResources as KatanaResources
import NodegraphAPI as NodegraphAPI
import PyQt5.QtCore as QtCore
import PyQt5.QtGui as QtGui
import typing
from PyUtilModule.VirtualKatana import Widgets as Widgets
from UI4.Widgets.ToolbarButton import ToolbarButton
from _typeshed import Incomplete
from typing import ClassVar, Set, Tuple

class NodeGoUpButton(ToolbarButton):
    _NodeGoUpButton__pixmaps: ClassVar[dict] = ...
    def __init__(self, setCurrentNodeViewCallback: typing.Optional[typing.Callable] = ..., parent: Incomplete | None = ...) -> None: ...
    def _NodeGoUpButton__loadResources(self): ...
    def _NodeGoUpButton__on_clicked(self): ...
    def setCurrentNodeView(self, node: NodegraphAPI.Node): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import PyQt5.QtGui
import QT4FormWidgets as QT4FormWidgets
import QT4FormWidgets.InputWidgets
import PyQt5.QtCore as QtCore
from UI4.Widgets.FilterPopups import ShaderFilterPopup as ShaderFilterPopup
from typing import Set, Tuple

class ShaderComboBox(QT4FormWidgets.InputWidgets.InputComboBox):
    def __init__(self, parent: QT4FormWidgets.FormWidget) -> None: ...
    def _ShaderComboBox__on_shaderFilterPopup_hide(self): ...
    def _ShaderComboBox__on_shaderFilterPopup_itemChosen(self, text, meta): ...
    def _ShaderComboBox__on_shaderFilterPopup_show(self): ...
    def keyPressEvent(self, event: PyQt5.QtGui.QKeyEvent): ...
    def showPopup(self): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import PyFCurve as PyFCurve
import PyQt5.QtWidgets
import QT4FormWidgets as QT4FormWidgets
import QT4Widgets as QT4Widgets
import QTFCurve as QTFCurve
import PyQt5.QtCore as QtCore
import PyQt5.QtWidgets as QtWidgets
import UI4 as UI4
import UI4.FormMaster.Editors.BaseRamp
import Utils as Utils
import collections
from UI4.FormMaster.Editors.BaseRamp import BaseRampFormWidget as BaseRampFormWidget
from UI4.FormMaster.Editors.SortableAttributeDropEditor import RegisterCoPolicyWidgetHintKey as RegisterCoPolicyWidgetHintKey
from _typeshed import Incomplete
from typing import ClassVar, Set, Tuple

class FloatRampFormWidget(UI4.FormMaster.Editors.BaseRamp.BaseRampFormWidget):
    DefaultMaxPosition: ClassVar[float] = ...
    DefaultMaxValue: ClassVar[float] = ...
    DefaultMinPosition: ClassVar[float] = ...
    DefaultMinValue: ClassVar[float] = ...
    Presets: ClassVar[collections.OrderedDict] = ...
    ValuesTypeString: ClassVar[str] = ...
    def __init__(self, parent, policy, factory) -> None: ...
    @staticmethod
    def Fit(value, inmin, inmax, outmin, outmax): ...
    def _FloatRampFormWidget__on_interpMenu_aboutToShow(self): ...
    def _FloatRampFormWidget__on_presetAction_triggered(self): ...
    def _FloatRampFormWidget__positionPolicyEvent(self, valuePolicyEvent): ...
    def _FloatRampFormWidget__updateUnderlyingParameters(self): ...
    def _FloatRampFormWidget__valuePolicyEvent(self, valuePolicyEvent): ...
    def _addControlWidgets(self, knots: list[tuple], interpolator: str, interpolatorOptions: list[str], layout: PyQt5.QtWidgets.QHBoxLayout): ...
    def _lockChanged(self, state: bool): ...
    def _updateControlWidgets(self, knots: list[tuple], interpolator: str): ...
    def rampChanged(self, final: bool = ...): ...
    def selectCurve(self): ...
    def selectedKeyframes(self) -> list[PyFCurve.Keyframe]: ...
    def updateInputWidgets(self): ...

class FloatRampWidget(PyQt5.QtWidgets.QWidget):
    def __init__(self, knots, interp, minX, maxX, minY, maxY, parent: Incomplete | None = ...) -> None: ...
    def contextMenuEvent(self, e): ...
    def getCurves(self): ...
    def getFcurve(self): ...
    def getGraphWidget(self): ...
    def getInterp(self): ...
    def getKeyframes(self): ...
    def replaceCurve(self, knots: list[tuple]): ...
    def selectCurve(self): ...
    def setInterp(self, value): ...

class SelectionObserver(PyFCurve.SelectionObserver):
    def __init__(self, parent) -> None: ...
    def deselected(self, obj: PyFCurve.FCurve | PyFCurve.Segment | PyFCurve.Keyframe | PyFCurve.Tangent): ...
    def selected(self, obj: PyFCurve.FCurve | PyFCurve.Segment | PyFCurve.Keyframe | PyFCurve.Tangent): ...

class SimpleObserver(PyFCurve.FCurveObserver):
    def __init__(self, parent) -> None: ...
    def endValueChange(self, curve): ...
    def intermediateValueChange(self, curve): ...
    def setMuted(self, muted): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import PyFnAttribute as FnAttribute
import PyQt5.QtCore
import PyQt5.QtGui
import PyQt5.QtWidgets
import QT4Color as QT4Color
import QT4FormWidgets as QT4FormWidgets
import PyQt5.QtCore as QtCore
import PyQt5.QtGui as QtGui
import PyQt5.QtWidgets as QtWidgets
import UI4.Widgets.SceneGraphView.Bridge
from UI4.Widgets.SceneGraphView.ColumnDataType import RegisterDataType as RegisterDataType
from UI4.Widgets.SceneGraphView.ItemDelegates.BaseItemDelegate import BaseItemDelegate as BaseItemDelegate
from UI4.Widgets.SceneGraphView.ItemDelegates.NameItemDelegate import NameItemDelegate as NameItemDelegate
from _typeshed import Incomplete
from typing import ClassVar, Set, Tuple

class SceneGraphTabColorColumnDelegate(BaseItemDelegate):
    def __init__(self, bridge: Bridge[UI4.Widgets.SceneGraphView.Bridge.Bridge], treeWidget: PyQt5.QtWidgets.QTreeWidget, parent: Incomplete | None = ...) -> None: ...
    def paint(self, painter: PyQt5.QtGui.QPainter, option: PyQt5.QtWidgets.QStyleOptionViewItem, index: PyQt5.QtCore.QModelIndex): ...

class SceneGraphTabCustomColumnDelegate(BaseItemDelegate):
    ColorSwatchWidth: ClassVar[int] = ...
    Margin: ClassVar[int] = ...
    MaximumColumnWidth: ClassVar[None] = ...
    Padding: ClassVar[int] = ...
    def __init__(self, bridge: Bridge[UI4.Widgets.SceneGraphView.Bridge.Bridge], treeWidget: PyQt5.QtWidgets.QTreeWidget, parent: Incomplete | None = ...) -> None: ...
    def _SceneGraphTabCustomColumnDelegate__getDisplayValue(self, modelValues: list[PyQt5.QtCore.QVariant]) -> str: ...
    def _SceneGraphTabCustomColumnDelegate__getModelValues(self, index: PyQt5.QtCore.QModelIndex) -> list[PyQt5.QtCore.QVariant | None]: ...
    def _drawColorSwatch(self, painter: PyQt5.QtGui.QPainter, values: list[PyQt5.QtCore.QVariant], parentRect: PyQt5.QtCore.QRect, width: int) -> PyQt5.QtCore.QRect: ...
    def _drawLabel(self, painter, values, parentRect): ...
    def _variantsToFloats(self, values): ...
    def isColorSwatchEnabled(self) -> bool: ...
    def isFilmlookEnabled(self) -> bool: ...
    def isLabelsEnabled(self) -> bool: ...
    def paint(self, painter: PyQt5.QtGui.QPainter, option: PyQt5.QtWidgets.QStyleOptionViewItem, index: PyQt5.QtCore.QModelIndex): ...
    def setColorSwatchEnabled(self, colorSwatchEnabled: bool): ...
    def setFilmlookEnabled(self, filmlookEnabled: bool): ...
    def setLabelsEnabled(self, labelsEnabled: bool): ...

class SceneGraphTabNameItemDelegate(NameItemDelegate):
    def initStyleOption(self, option: PyQt5.QtWidgets.QStyleOptionViewItem, index: PyQt5.QtCore.QModelIndex): ...
    def shouldShowPinning(self) -> bool: ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import PyQt5.QtGui
import PyQt5.QtWidgets
import QT4FormWidgets as QT4FormWidgets
import QT4FormWidgets.InputWidgets
import QT4Widgets as QT4Widgets
import PyQt5.QtCore as QtCore
import ResolutionTable as ResolutionTable
from typing import ClassVar, Set, Tuple

class ResolutionComboBox(QT4FormWidgets.InputWidgets.InputComboBox):
    _ResolutionComboBox__Groups: ClassVar[dict] = ...
    def __init__(self, parent: PyQt5.QtWidgets.QWidget | None) -> None: ...
    @staticmethod
    def _ResolutionComboBox__GroupsCallback(value: str, name: str, meta: object) -> bool: ...
    def _ResolutionComboBox__on_filterablePopup_aboutToShow(self): ...
    def _ResolutionComboBox__on_filterablePopup_hide(self): ...
    def _ResolutionComboBox__on_filterablePopup_itemChosen(self, text, meta): ...
    def _ResolutionComboBox__on_filterablePopup_show(self): ...
    def getResolutionTableEntry(self) -> ResolutionTable.ResolutionTableEntry: ...
    def getSettings(self) -> tuple[str, int, int, float]: ...
    def getSize(self) -> None: ...
    def keyPressEvent(self, event: PyQt5.QtGui.QKeyEvent): ...
    def setSelectionByName(self, name: str): ...
    def setSelectionBySize(self, width: int, height: int, aspectRatio: float = ...): ...
    def showPopup(self): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import UI4.Tabs.UVPanel.FaceSelectionHelper as FaceSelectionHelper
import PyFnAttribute as FnAttribute
import PyFnGeolib as FnGeolib
import UI4.Tabs.UVPanel.Layers as Layers
import NodegraphAPI as NodegraphAPI
import Nodes2DAPI as Nodes2DAPI
import Nodes3DAPI as Nodes3DAPI
import QT4GLLayerStack as QT4GLLayerStack
import QT4GLLayerStack.LayerStack
import PyQt5.QtGui as QtGui
import PyQt5.QtWidgets as QtWidgets
import Utils as Utils
from PyUtilModule.VirtualKatana import FaceSelectionManager as FaceSelectionManager, ScenegraphManager as ScenegraphManager
from QT4GLLayerStack.LayerStack import LayerStack
from _typeshed import Incomplete
from typing import Set, Tuple

class BGLayer(QT4GLLayerStack.LayerStack.Layer):
    def __init__(self, *args, **kwargs) -> None: ...
    def getTextureInfo(self): ...
    def loadTextureCallback(self, image: Incomplete | None = ..., texturePath: Incomplete | None = ..., tileLocation: tuple = ...): ...
    def paintGL(self): ...
    def paintTexture(self, textureName, x, y): ...
    def repaintGL(self, *args, **kwargs): ...
    def setStencilBuffer(self, enabled): ...
    def setTexture(self, file): ...
    def setUVTileSequenceClass(self, sequenceClass): ...

class ClearToBaseLayer(QT4GLLayerStack.LayerStack.Layer):
    def __init__(self, name: Incomplete | None = ..., visible: bool = ..., interactive: bool = ..., enabled: bool = ...) -> None: ...
    def paintGL(self): ...

class GeoLayer(QT4GLLayerStack.LayerStack.Layer):
    def __init__(self, *args, **kwargs) -> None: ...
    def _GeoLayer__clearCache(self, *args, **kwargs): ...
    def _GeoLayer__drawPillPoly(self, x1, y1, x2, y2): ...
    def _GeoLayer__generateGeometryList(self, selectedFaces: bool = ...): ...
    def clearSelection(self): ...
    def computeSelection(self, rect, shift, control): ...
    def filterSearchTree(self): ...
    def forceRebuild(self, selectedOnly: bool = ...): ...
    def generateStencilBuffer(self, bufferEnabled): ...
    def getBounds(self): ...
    def getClient(self): ...
    def getRuntime(self): ...
    def getSelectedFaces(self): ...
    def getSelectedPaths(self): ...
    def getSelection(self, clearGeomCache: bool = ...): ...
    def getSelectionBounds(self): ...
    def getTexture(self, textureID): ...
    def getTextureIndex(self): ...
    def paintGL(self): ...
    def setAdditionalTextures(self, file): ...
    def setIsolateSelection(self, isolateSelection): ...
    def setSelectedFaces(self, faces): ...
    def setUpdateTexturesCallback(self, cb): ...
    def setUpdateTilesCallback(self, cb): ...

class OverlayLayer(QT4GLLayerStack.LayerStack.Layer):
    def __init__(self, *args, **kwargs) -> None: ...
    def _OverlayLayer__getTextLabel(self, u, v, textureInfo): ...
    def getSettingState(self, name): ...
    def paintGL(self): ...
    def setSettingState(self, name, value): ...
    def setTextureInfoCallback(self, function): ...

class UVWidget(LayerStack):
    def __init__(self, parent) -> None: ...
    def _UVWidget__addImplicitResolvers(self, txn, opId, viewedNode): ...
    def _UVWidget__buildLayers(self): ...
    def _UVWidget__checkForLocationEvents(self): ...
    def _UVWidget__connectToEvents(self): ...
    def _UVWidget__createTerminalOp(self): ...
    def _UVWidget__faceSelectionChanged(self, args): ...
    def _UVWidget__idle_callback(self, *args, **kwargs): ...
    def _UVWidget__nodeSetViewed(self, args): ...
    def _UVWidget__scenegraphManager_selectionChanged_callback(self, eventType, eventID, **kwargs): ...
    def _UVWidget__terminalOpCallback(self, port: NodegraphAPI.Port, graphState: NodegraphAPI.GraphState, op, transaction): ...
    def _UVWidget__updateFaceSelectionManager(self): ...
    def computeSelection(self, rect, shift, control): ...
    def event(self, event): ...
    def frame(self): ...
    def frameAll(self): ...
    def frameTile(self, uvPosition: tuple = ...): ...
    def getOverlaySetting(self, name): ...
    def getSelectedFaces(self): ...
    def getSelectedPaths(self): ...
    def getSelection(self): ...
    def getStencilBuffer(self): ...
    def getTextureIndex(self): ...
    def getWorldBounds(self): ...
    def hideEvent(self, event): ...
    def paintGL(self): ...
    def requestRebuild(self): ...
    def resizeGL(self, width, height): ...
    def selectTexture(self, textureID): ...
    def setIsolateSelection(self, isolateSelection): ...
    def setOverlaySetting(self, name, value): ...
    def setStencilBuffer(self, enabled): ...
    def setTexture(self, texturePath): ...
    def setUVTileSequenceClass(self, sequenceClass): ...
    def setUpdateTexturesCallback(self, cb): ...
    def setUpdateTilesCallback(self, cb): ...
    def showEvent(self, event): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import NodegraphAPI
import UI4.Widgets.SceneGraphView.ViewLink
import UI4.Widgets.SceneGraphView.SceneGraphHandle
import UI4.Widgets.SceneGraphView.ViewLink
import Utils as Utils
import abc
import typing
from Utils.Decorators import deprecated as deprecated
from _typeshed import Incomplete
from typing import ClassVar, Set, Tuple

class Bridge(abc.ABC):
    _abc_impl: ClassVar[_abc_data] = ...
    __abstractmethods__: ClassVar[frozenset] = ...
    def addAttributeDataItemType(self, itemTypeClass: type): ...
    def addColumn(self, columnName): ...
    def addColumnSet(self, columnSetName): ...
    def addRule(self, ruleName): ...
    def addTopLevelLocation(self, topLevelLocationPath: str, index: Incomplete | None = ...): ...
    def applyColumnVisibilitySettings(self, columnVisibility: dict): ...
    def applyColumnWidthsSettings(self, columnWidths: dict): ...
    def beginColumnConfiguration(self): ...
    def clearRuleCriteria(self, ruleName, triggerEvaluation: bool = ...): ...
    def clearTopLevelLocations(self): ...
    def collapseLocation(self, handle: SceneGraphHandle[UI4.SceneGraphView.SceneGraphHandle.SceneGraphHandle], topLevelHandle: SceneGraphHandle[UI4.SceneGraphView.SceneGraphHandle.SceneGraphHandle], recursive: bool = ...): ...
    def columnResized(self, columnIndex, newColumnWidth): ...
    def columnTitleClicked(self, columnIndex, columnTitleRect): ...
    def endColumnConfiguration(self): ...
    def expandLocation(self, handle: SceneGraphHandle[UI4.SceneGraphView.SceneGraphHandle.SceneGraphHandle], topLevelHandle: SceneGraphHandle[UI4.SceneGraphView.SceneGraphHandle.SceneGraphHandle], recursive: bool = ..., stopTypes: Incomplete | None = ...): ...
    def fontChanged(self): ...
    def freeze(self): ...
    def getAttributeDataItemTypes(self) -> list[type]: ...
    def getColumnByIndex(self, columnIndex: int, visibleOnly: bool = ...) -> UI4.SceneGraphView.SceneGraphViewColumn.SceneGraphColumn | None: ...
    def getColumnByName(self, columnName: str) -> UI4.SceneGraphView.SceneGraphViewColumn.SceneGraphColumn | None: ...
    def getColumnDataType(self, columnIndex: int) -> int: ...
    def getColumnPresetManager(self) -> ColumnPresetManager[UI4.SceneGraphView.ColumnPresetManager.ColumnPresetManager]: ...
    def getColumnVisibilitySettings(self) -> dict: ...
    def getColumnWidthsSettings(self) -> dict: ...
    def getCurrentRules(self): ...
    def getDisplayData(self, handle, topLevelHandle, columnIndex) -> str | float | list: ...
    def getNumberOfChildren(self, locationPath: str) -> str | None: ...
    def getNumberOfColumns(self): ...
    def getNumberOfTopLevelLocations(self) -> int: ...
    def getOverrideNodeAndParameterName(self, locationPath: str, attributeName: str) -> None: ...
    def getParent(self, locationPath: str) -> str | None: ...
    def getPotentialChildren(self, locationPath: str) -> list[str]: ...
    def getResolvedAttributeName(self, handle: SceneGraphHandle[UI4.Widgets.SceneGraphView.SceneGraphHandle.SceneGraphHandle], attributeName: str) -> tuple[bool, str]: ...
    def getRootColumnSet(self) -> SceneGraphColumnSet[UI4.SceneGraphView.SceneGraphViewColumn.SceneGraphColumnSet]: ...
    def getRuleByName(self, ruleName): ...
    def getRuleMatchInversion(self): ...
    def getSceneGraphAttributes(self, locationPath, callback: Incomplete | None = ..., oneShot: bool = ...): ...
    def getSceneGraphChildren(self, parentLocationPath, callback: Incomplete | None = ..., oneShot: bool = ...): ...
    def getTitleText(self, columnIndex: int) -> str: ...
    def getTopLevelDisplayIndex(self, handle): ...
    def getTopLevelLocations(self) -> list[str]: ...
    def getViewLink(self) -> ViewLink[UI4.Widgets.SceneGraphView.ViewLink.ViewLink] | None: ...
    def hideColumnSet(self, columnSetName: str): ...
    def interruptClientProcessing(self): ...
    def isEditable(self, columnIndex): ...
    def isFrozen(self) -> bool: ...
    def isNameColumn(self, columnIndex): ...
    def isProcessing(self) -> bool: ...
    def isStopping(self) -> bool: ...
    def locationNamesPolished(self) -> bool: ...
    def matchRules(self, handle, topLevelHandle): ...
    def moveColumn(self, columnIndex, newColumnIndex): ...
    def removeRule(self, ruleName): ...
    def removeTopLevelLocation(self, topLevelLocationPath: str): ...
    def renameRule(self, oldName, newName): ...
    def resetAttributeDataItemTypes(self): ...
    def setKeyPressEventCallback(self, callback: typing.Callable): ...
    def setLocationActive(self, locationPath: str): ...
    def setLocationAddedOrUpdatedCallback(self, callback): ...
    def setLocationDataProcessedCallback(self, callback): ...
    def setLocationNamePolishCallback(self, callback: typing.Callable): ...
    def setLocationNamesPolished(self, locationNamesPolished): ...
    def setLocationRemovedCallback(self, callback): ...
    def setMaxEventsToProcess(self, maxEventsToProcess: int): ...
    def setRuleComment(self, ruleName, comment): ...
    def setRuleCriteria(self, ruleName, criteriaName, criteria): ...
    def setRuleEnabled(self, ruleName, isEnabled): ...
    def setRuleEvaluator(self, ruleName, evaluator, useDefaultCriteria: bool = ...): ...
    def setRuleInverted(self, ruleName, isInverted): ...
    def setRuleMatchInversion(self, invert): ...
    def setRuleMatchPolicy(self, policy): ...
    def setRuleShowChildren(self, ruleName, showChildren): ...
    def setRuleTarget(self, ruleName, column): ...
    def setStopRecursiveExpandAttributeNames(self, attributeNames: list[str]): ...
    def setStopRecursiveExpandLocationTypes(self, locationTypes: list[str]): ...
    def setTerminalOps(self, terminalOpArgs): ...
    def setTopLevelDisplayIndex(self, locationPath: str, index: int): ...
    def setTopLevelLocations(self, topLevelLocations: list[str]): ...
    def setUpdateViewportAutomatically(self, updateViewportAutomatically: bool): ...
    def setViewLink(self, viewLink): ...
    def setViewNode(self, viewNode: NodegraphAPI.Node | None, applyImplicitResolvers: bool = ..., extraImplicitResolverOpArgs: Incomplete | None = ...): ...
    def setViewOp(self, viewOp, applyImplicitResolvers: bool = ..., extraImplicitResolverOpArgs: Incomplete | None = ...): ...
    def showColumnSet(self, columnSetName: str): ...
    def thaw(self): ...
    def topLevelLocationRenamed(self, oldLocationPath, newLocationPath): ...
    def updateTerminalOps(self, terminalOpArgs, txn: Incomplete | None = ...): ...
    def updateViewLink(self): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import NodegraphAPI as NodegraphAPI
import Nodes2DAPI_cmodule as Nodes2DAPI_cmodule
import Utils as Utils
import typing
from _typeshed import Incomplete
from typing import Set, Tuple

_mtrsCache: dict

class _frame:
    def __init__(self, x0, y0, x1, y1) -> None: ...

def GetDisplayWindow(port: int = ..., node: typing.Optional[NodegraphAPI.Node] = ..., time: Incomplete | None = ...): ...
def GetXform(node: typing.Optional[NodegraphAPI.Node] = ..., time: Incomplete | None = ...): ...
def GetXformR(node: typing.Optional[NodegraphAPI.Node] = ..., time: Incomplete | None = ...): ...
def GetXformS(node: typing.Optional[NodegraphAPI.Node] = ..., time: Incomplete | None = ...): ...
def GetXformT(node: typing.Optional[NodegraphAPI.Node] = ..., time: Incomplete | None = ...): ...
def _GetXformMTRS(node: typing.Optional[NodegraphAPI.Node] = ..., time: Incomplete | None = ...): ...
def __clearMTRSCache(*args, **kwargs): ...
def _getSourcePortAndTime(portName, node: NodegraphAPI.Node, time): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import AssetAPI as AssetAPI
import Callbacks as Callbacks
import ConfigurationAPI_cmodule as Configuration
import Utils.EventModule as EventModule
import NodegraphAPI
import NodegraphAPI_cmodule as NodegraphAPI_cmodule
import PyXmlIO as PyXmlIO
import Utils as Utils
import NodegraphAPI.Version as Version
import NodegraphAPI.Xio as Xio
from NodegraphAPI_cmodule import __root_node as __root_node
from _typeshed import Incomplete
from typing import Set, Tuple

def CheckNodeType(node: NodegraphAPI.Node, stringOrClassTypes): ...
def GetAutoKeyAll() -> bool: ...
def GetCurrentTime() -> float: ...
def GetInTime() -> float: ...
def GetKatanaSceneName() -> str | None: ...
def GetNodegraphVersionString(xmlString) -> str: ...
def GetOriginalProjectAssetID() -> str: ...
def GetOriginalSourceFile() -> str: ...
def GetOutTime() -> float: ...
def GetProjectAssetID() -> str: ...
def GetProjectDir() -> str: ...
def GetProjectFile() -> str: ...
def GetRootNode() -> NodegraphAPI.GroupNode: ...
def GetSourceFile() -> str: ...
def GetTimeIncrement() -> float: ...
def GetWorkingInTime() -> float: ...
def GetWorkingOutTime() -> float: ...
def IsLoading() -> bool: ...
def IsLoadingAsync() -> bool: ...
def LoadXmlFromFile(fileName): ...
def LoadXmlFromString(stringData): ...
def NeedsRedraw(): ...
def NewNodegraph(): ...
def RegisterNodegraphChangedEvents(): ...
def ResetAllFloatingCurves(): ...
def SetAutoKeyAll(autoKeyAll: bool): ...
def SetCurrentTime(newTime: float, final: bool = ...): ...
def SetInTime(inTime: float, final: bool = ...): ...
def SetKatanaSceneName(katanaSceneName): ...
def SetNeedsRedraw(state): ...
def SetOriginalProjectAssetID(originalProjectAssetID): ...
def SetOriginalSourceFile(originalSourceName): ...
def SetOutTime(outTime: float, final: bool = ...): ...
def SetProjectAssetID(filenameOrAssetID: str): ...
def SetRootNode(newNode): ...
def SetSourceFile(filenameOrAssetID: str): ...
def SetTimeIncrement(incTime: float, final: bool = ...): ...
def SetWorkingInTime(workingInTime: float, final: bool = ...): ...
def SetWorkingOutTime(workingOutTime: float, final: bool = ...): ...
def UpdatePluginsFromGlobals(): ...
def WriteKatanaFile(filename: str, element: PyXmlIO.Element, compress: bool = ..., archive: bool = ..., opaqueParams: Incomplete | None = ...): ...
def WriteKatanaString(element: PyXmlIO.Element, compress: bool = ..., archive: bool = ..., opaqueParams: Incomplete | None = ...) -> str: ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import CatalogAPI as CatalogAPI
import UI4.Util.CatalogEventReceiver as CatalogEventReceiver
import Nodes2DAPI as Nodes2DAPI
import PyQt5.QtCore
import PyQt5.QtCore as QtCore
import PyQt5.QtGui as QtGui
import PyQt5.QtWidgets as QtWidgets
import PyUtilModule.RenderManager as RenderManager
import UI4.Util.TextureManager as TextureManager
import Utils as Utils
from UI4.Util.CatalogEventReceiver import _CatalogEventReceiver as _CatalogEventReceiver
from typing import Set, Tuple

class RegenerateThumbnailThread(PyQt5.QtCore.QThread):
    def __init__(self) -> None: ...
    @staticmethod
    def _RegenerateThumbnailThread__nodeGraph_loadBegin_CB(*_args, **_kwargs): ...
    def run(self): ...
    def stopThread(self): ...

def EnableCatalogEventHandling(): ...
def RegenerateThumbnail(catalogItem): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import PyFnGeolibProducers as FnGeolibProducers
import UI4.Util.IconManager as IconManager
import KatanaResources as KatanaResources
import UI4.FormMaster.Editors.NodeGroup as NodeGroup
import NodegraphAPI as NodegraphAPI
import PyQt5.QtCore
import PyQt5.QtWidgets
import QT4Browser as QT4Browser
import QT4FormWidgets as QT4FormWidgets
import QT4FormWidgets.PythonValuePolicy
import QT4FormWidgets.ValuePolicy
import QT4Widgets.SortableTreeWidget
import PyQt5.QtCore as QtCore
import PyQt5.QtGui as QtGui
import PyQt5.QtWidgets as QtWidgets
import UI4.Util.ScenegraphIconManager as ScenegraphIconManager
import UI4.Widgets.SortablePanel as SortablePanel
import UI4 as UI4
import UI4.FormMaster.Editors.NodeGroup
import UI4.Widgets.SortablePanel
import Utils as Utils
from QT4Widgets.SortableTreeWidget import SortableTreeWidgetItemDelegate as SortableTreeWidgetItemDelegate
from typing import ClassVar, Set, Tuple

class ChildMaskPolicy(QT4FormWidgets.ValuePolicy.ValuePolicyProxy):
    def __init__(self, policy, childIncludeSet) -> None: ...
    def getChildByName(self, name): ...
    def getChildren(self): ...
    def getWidgetHints(self): ...

class LookFileInNodePanel(UI4.Widgets.SortablePanel.SortablePanelBase):
    CHECKBOX_COLUMN: ClassVar[int] = ...
    NAME_COLUMN: ClassVar[int] = ...
    _LookFileInNodePanel__pixmaps: ClassVar[dict] = ...
    def __init__(self, parent, node: NodegraphAPI.Node, factory) -> None: ...
    def _LookFileInNodePanel__actionsMenuAboutToShow(self): ...
    def _LookFileInNodePanel__filterUpdate(self, filterText, filterType): ...
    def _LookFileInNodePanel__getSelectedItems(self): ...
    def _LookFileInNodePanel__listMousePressEvent(self, event): ...
    def _LookFileInNodePanel__setStatusLabel(self, enabled, shown, hidden): ...
    def _LookFileInNodePanel__toggleItemState(self, item): ...
    def _LookFileInNodePanel__updateMaterialList(self): ...
    def getNode(self) -> NodegraphAPI.Node: ...
    def updatePanel(self): ...

class LookFileInStackEditor(UI4.Widgets.SortablePanel.SortablePanelFormWidget):
    _LookFileInStackEditor__EVENTNAMES: ClassVar[tuple] = ...
    def __init__(self, parent, node: NodegraphAPI.Node, factory) -> None: ...
    def _LookFileInStackEditor__collapsedEventHander(self, args): ...
    def _LookFileInStackEditor__registerEvents(self): ...
    def _LookFileInStackEditor__unregisterEvents(self): ...
    def _freeze(self): ...
    def _thaw(self): ...
    def panelDeleted(self, index): ...
    def panelReordered(self, oldPos, newPos): ...
    def updatePanels(self): ...

class MaterialStackFormWidget(UI4.FormMaster.Editors.NodeGroup.NodeGroupFormWidget):
    _MaterialStackFormWidget__EVENTNAMES: ClassVar[tuple] = ...
    _MaterialStackFormWidget__pixmaps: ClassVar[dict] = ...
    def __init__(self, parent, policy, factory) -> None: ...
    def _MaterialStackFormWidget__aboutToDragCallback(self, items, dragObject): ...
    def _MaterialStackFormWidget__addChildMaterial(self): ...
    def _MaterialStackFormWidget__addGeometryMaterial(self): ...
    def _MaterialStackFormWidget__addLightMaterial(self): ...
    def _MaterialStackFormWidget__addLookFileMaterial(self): ...
    def _MaterialStackFormWidget__addLookFileReference(self): ...
    def _MaterialStackFormWidget__addMaterial(self): ...
    def _MaterialStackFormWidget__addMaterialItem(self, parentItem, node: NodegraphAPI.Node, childList, selectedNodes, openNodes, kstdRefDict, selectedItems): ...
    def _MaterialStackFormWidget__addMenuAboutToShow(self): ...
    def _MaterialStackFormWidget__buildParameterEditorForChildNode(self, widgetParent, childNode, addToCache: bool = ...): ...
    def _MaterialStackFormWidget__collapsedEventHander(self, args): ...
    def _MaterialStackFormWidget__deferredSelectionChanged(self): ...
    def _MaterialStackFormWidget__deleteMaterial(self, confirm: bool = ...): ...
    def _MaterialStackFormWidget__dragMoveEventCallback(self, event, parent, index, callbackRecord): ...
    def _MaterialStackFormWidget__dropEventCallback(self, event, parent, index): ...
    def _MaterialStackFormWidget__duplicateMaterial(self): ...
    def _MaterialStackFormWidget__findAboutToShow(self): ...
    def _MaterialStackFormWidget__findItemChosen(self, text, meta): ...
    def _MaterialStackFormWidget__freezeToggled(self, state): ...
    def _MaterialStackFormWidget__getActiveNodes(self): ...
    def _MaterialStackFormWidget__getNode(self): ...
    def _MaterialStackFormWidget__ignoreMaterial(self): ...
    def _MaterialStackFormWidget__initMembers(self): ...
    def _MaterialStackFormWidget__listMousePressEvent(self, event): ...
    def _MaterialStackFormWidget__listViewKeyPressCallback(self, event): ...
    def _MaterialStackFormWidget__loadResources(self): ...
    def _MaterialStackFormWidget__reapEditorCache(self): ...
    def _MaterialStackFormWidget__registerEvents(self): ...
    def _MaterialStackFormWidget__selectionChanged(self): ...
    def _MaterialStackFormWidget__tearOffParameters(self): ...
    def _MaterialStackFormWidget__unregisterEvents(self): ...
    def _MaterialStackFormWidget__updateListContents(self): ...
    def _buildFindWidget(self, layout): ...
    def _freeze(self): ...
    def _thaw(self): ...
    def getSelectedMaterialPaths(self): ...

class MaterialTreeDelegate(QT4Widgets.SortableTreeWidget.SortableTreeWidgetItemDelegate):
    def paint(self, painter, option, index): ...

class MaterialTreeItem(QT4Widgets.SortableTreeWidget.SortableTreeWidgetItem):
    def isDraggable(self): ...

class NameParameterEditor(PyQt5.QtWidgets.QFrame):
    disableLock: ClassVar[PyQt5.QtCore.pyqtSignal] = ...
    def __init__(self, parent, node: NodegraphAPI.Node, factory, showLockButton: bool = ...) -> None: ...
    def _NameParameterEditor__clicked(self): ...
    def getValuePolicy(self): ...

class PolicyShim(QT4FormWidgets.PythonValuePolicy.PythonGroupPolicy):
    def __init__(self, node: NodegraphAPI.Node) -> None: ...
    def getNode(self) -> NodegraphAPI.Node: ...
    def getParameter(self): ...

class ReferencedMaterialTreeItem(MaterialTreeItem):
    ITALIC_FONT: ClassVar[None] = ...
    def __init__(self, *args, **kwds) -> None: ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import WorkQueue as WorkQueue
from typing import Set, Tuple

class QuadTree:
    def __init__(self) -> None: ...
    def _QuadTree__filterTree(self): ...
    def addContent(self, items): ...
    def clear(self): ...
    def filterTreeBegin(self): ...
    def getContentsInRect(self, rect): ...
    def setBounds(self, rect): ...

class QuadTreeNode:
    def __init__(self, bounds, depth: int = ...) -> None: ...
    def GetMaxDepth(self, depth): ...
    def _QuadTreeNode__distributeToChildren(self, items): ...
    def addContent(self, item): ...
    def clear(self): ...
    def getBounds(self): ...
    def getContentsInRect(self, rect): ...
    def printContents(self, depth: int = ...): ...
    def setBounds(self, rect): ...

def rectContainsPoint(rect, point): ...
def rectsIntersect(rect1, rect2): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import DrawingModule as DrawingModule
import PyUtilModule.KatanaFile as KatanaFile
import NodegraphAPI as NodegraphAPI
import PyQt5.QtCore
import QT4FormWidgets as QT4FormWidgets
import QT4Widgets as QT4Widgets
import PyQt5.QtCore as QtCore
import PyQt5.QtGui as QtGui
import PyQt5.QtWidgets as QtWidgets
import UI4 as UI4
import Utils as Utils
from QT4FormWidgets.FormWidget import FormWidget
from QT4Widgets.FilterablePopupButton import FilterablePopupButton
from _typeshed import Incomplete
from typing import ClassVar, Set, Tuple

class FindButton(FilterablePopupButton):
    _FindButton__SELECTBUTTONTEXT: ClassVar[str] = ...
    selectAllClicked: ClassVar[PyQt5.QtCore.pyqtSignal] = ...
    def __init__(self, icon, iconSize, *args) -> None: ...
    def _FindButton__aboutToShow(self): ...
    def _FindButton__findFilterTypeCallback(self, name, meta, matchstring): ...
    def _FindButton__selectAllClicked(self): ...
    def _FindButton__setSelectButtonState(self, state: bool = ...): ...

class GroupStackFormWidget(FormWidget):
    pressed: ClassVar[PyQt5.QtCore.pyqtSignal] = ...
    selectionChanged: ClassVar[PyQt5.QtCore.pyqtSignal] = ...
    def __init__(self, parent, policy, factory) -> None: ...
    def _GroupStackFormWidget__addButtonClicked(self): ...
    def _GroupStackFormWidget__addNodeMenuChosen(self, name, meta): ...
    def _GroupStackFormWidget__buildParameterEditorForChildNode(self, widgetParent, childNode): ...
    def _GroupStackFormWidget__clearParameterDisplayLayout(self): ...
    def _GroupStackFormWidget__contextMenuRequestedCB(self, pos): ...
    def _GroupStackFormWidget__copySelectedEntries(self): ...
    def _GroupStackFormWidget__cutSelectedEntries(self): ...
    def _GroupStackFormWidget__deleteSelectedEntries(self): ...
    def _GroupStackFormWidget__duplicateSelectedEntries(self): ...
    def _GroupStackFormWidget__editSelected(self): ...
    def _GroupStackFormWidget__findItemChosen(self, nodeName, meta: Incomplete | None = ...): ...
    def _GroupStackFormWidget__findPopupAboutToShow(self): ...
    def _GroupStackFormWidget__findPopupSelectAllClicked(self, nodeNames, addFlag): ...
    def _GroupStackFormWidget__freezeToggled(self, state): ...
    def _GroupStackFormWidget__getNode(self): ...
    def _GroupStackFormWidget__idle_callback(self, *args, **kwargs): ...
    def _GroupStackFormWidget__itemSelectionChangedCB(self): ...
    def _GroupStackFormWidget__node_setByPassedCB(self, eventType, eventID, **kwargs): ...
    def _GroupStackFormWidget__node_setColorCB(self, eventType, eventID, **kwargs): ...
    def _GroupStackFormWidget__node_setName(self, eventType, eventID, **kwargs): ...
    def _GroupStackFormWidget__node_setParentCB(self, eventType, eventID, **kwargs): ...
    def _GroupStackFormWidget__parameter_finalizeValueCB(self, eventType, eventID, **kwargs): ...
    def _GroupStackFormWidget__pasteEntries(self): ...
    def _GroupStackFormWidget__port_disconnectCB(self, eventType, eventID, **kwargs): ...
    def _GroupStackFormWidget__reapEditorCache(self): ...
    def _GroupStackFormWidget__rebuildParameterEditorForChildNode(self, childNode): ...
    def _GroupStackFormWidget__reorderItemsCallback(self, indexList, toIndex): ...
    def _GroupStackFormWidget__setDisplayOfChildType(self, childType): ...
    def _GroupStackFormWidget__setupEventHandlers(self, active): ...
    def _GroupStackFormWidget__stackListAboutToDragCallback(self, items, dragObject): ...
    def _GroupStackFormWidget__stackListDragEnterEventCallback(self, event): ...
    def _GroupStackFormWidget__stackListDragMoveEventCallback(self, event): ...
    def _GroupStackFormWidget__stackListDropEventCallback(self, event, toIndex): ...
    def _GroupStackFormWidget__stackListKeyPressCallback(self, event): ...
    def _GroupStackFormWidget__tearoffSelectedEntries(self): ...
    def _GroupStackFormWidget__toggleIgnoreOfSelectedEntries(self): ...
    def _GroupStackFormWidget__updateListContents(self): ...
    def _GroupStackFormWidget__updateListState(self): ...
    def _GroupStackFormWidget__viewContentsOfGroup(self): ...
    def _GroupStackFormWidget__viewSelected(self): ...
    def _buildTopAreaLayout(self, layout): ...
    def _freeze(self): ...
    def _getDisplayNameForChildNode(self, child): ...
    def _getRootParameterPolicyForChildNode(self, childNode): ...
    def _lockChanged(self, state: bool): ...
    def _thaw(self): ...
    def event(self, event): ...
    def getSelectedNodes(self): ...
    def isLiveGroupStack(self): ...
    def resizeEvent(self, event): ...

class LookThroughGroupStackFormWidget(GroupStackFormWidget):
    def _getDisplayNameForChildNode(self, child): ...
    def _getRootParameterPolicyForChildNode(self, childNode): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import PyQt5.QtWidgets
import QT4Color as QT4Color
import PyQt5.QtWidgets as QtWidgets
import UI4 as UI4
import UI4.FormMaster.GroupParameterPolicy
import Utils as Utils
from _typeshed import Incomplete
from typing import Set, Tuple

def GetCCArrayIndexHints(index, units): ...
def GetCCArrayIndexName(index): ...
def ShowColorPicker(colorPolicy: GroupParameterPolicy[UI4.FormMaster.GroupParameterPolicy.GroupParameterPolicy], widget: Incomplete | None = ...) -> PyQt5.QtWidgets.QDialog.DialogCode: ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import PyQt5.QtCore
import QT4FormWidgets as QT4FormWidgets
import QT4FormWidgets.Conditional
import PyQt5.QtCore as QtCore
import UI4.FormMaster.States as States
import Utils as Utils
from typing import ClassVar, Set, Tuple

class EndsWith(QT4FormWidgets.Conditional.ConditionalVisibilityOpBase):
    def __init__(self, targetPolicy, hints, prefix) -> None: ...
    def checkState(self): ...

class InputConnected(QT4FormWidgets.Conditional.ConditionalVisibilityOpBase):
    _InputConnected__PORTEVENTS: ClassVar[tuple] = ...
    conditionalStateEval: ClassVar[PyQt5.QtCore.pyqtSignal] = ...
    def __init__(self, targetPolicy, hints, prefix) -> None: ...
    def _InputConnected__notifyOfChange(self): ...
    def _InputConnected__portStatusChanged(self, args): ...
    def checkState(self): ...
    def freeze(self): ...
    def thaw(self): ...

class IsLocalAssignment(IsNodeDefault):
    def checkState(self): ...

class IsNodeDefault(QT4FormWidgets.Conditional.ConditionalVisibilityOpBase):
    conditionalStateEval: ClassVar[PyQt5.QtCore.pyqtSignal] = ...
    def __init__(self, targetPolicy, hints, prefix) -> None: ...
    def _IsNodeDefault__isNodeDefault(self, policy, recur: bool = ...): ...
    def _IsNodeDefault__targetPolicyStateChanged(self, event): ...
    def checkState(self): ...
    def freeze(self): ...
    def thaw(self): ...

class NumChildrenBase(QT4FormWidgets.Conditional.ConditionalVisibilityOpBase):
    conditionalStateEval: ClassVar[PyQt5.QtCore.pyqtSignal] = ...
    def __init__(self, targetPolicy, hints, prefix) -> None: ...
    def _NumChildrenBase__paramChildrenChange_CB(self, args): ...
    def _NumChildrenBase__setupEvents(self, enable): ...
    def _getNumChildren(self): ...
    def checkState(self): ...
    def freeze(self): ...
    def thaw(self): ...

class NumChildrenEqualTo(NumChildrenBase):
    def checkState(self): ...

class NumChildrenGreaterThanOrEqualTo(NumChildrenBase):
    def checkState(self): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import DrawingModule as DrawingModule
import NodegraphAPI as NodegraphAPI
import QT4GLLayerStack as QT4GLLayerStack
import QT4GLLayerStack.LayerStack
import PyQt5.QtCore as QtCore
import PyQt5.QtGui as QtGui
import PyQt5.QtWidgets as QtWidgets
import UI4 as UI4
from _typeshed import Incomplete
from typing import Set, Tuple

class NetworkMaterialNodeInteractionLayer(QT4GLLayerStack.LayerStack.Layer):
    def __init__(self, *args, **kwargs) -> None: ...
    def _NetworkMaterialNodeInteractionLayer__deletePortCallback(self, mId: Incomplete | None = ...): ...
    def _NetworkMaterialNodeInteractionLayer__renamePortCallback(self, menuid: int = ...): ...
    def processEvent(self, event): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import AssetAPI as AssetAPI
import ConfigurationAPI_cmodule as Configuration
import NodegraphAPI as NodegraphAPI
import PyUtilModule as PyUtilModule
import PyResolutionTableFn as ResolutionTable
from _typeshed import Incomplete
from typing import Set, Tuple

def ExportCatalogToDisk(items, filePath, imageFormat, imageOptions): ...
def GetCommonResolutionNameFromItems(items): ...
def GetFirstFrame(resolvedLocation): ...
def GetMemoryUsageMB(): ...
def GetNearestTimeInSequence(sequence, time): ...
def GetSequenceCreationTime(resolvedLocation): ...
def GetSequenceObj(resolvedLocation: str) -> FileSequence | None: ...
def GetUniqueImageNumberInDirectory(directory): ...
def ImportImageSequence(assetId, nodeName: str = ..., setPersistant: bool = ..., sampleRate: Incomplete | None = ...): ...
def IsPersistantCatalogAllowed(): ...
def ValidatePersistantDirectory(onlyTestExistance: bool = ...): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import PyQt5.QtCore
import PyQt5.QtWidgets
import PyXmlIO
import PyQt5.QtCore as QtCore
import PyQt5.QtWidgets as QtWidgets
import Utils as Utils
from _typeshed import Incomplete
from typing import ClassVar, Set, Tuple

class CatalogRenderGlobalsStatsWidget(PyQt5.QtWidgets.QTreeWidget):
    statsPresent: ClassVar[PyQt5.QtCore.pyqtSignal] = ...
    def __init__(self, parent: Incomplete | None = ...) -> None: ...
    def _CatalogRenderGlobalsStatsWidget__catalog_itemDelete_CB(self, eventType, eventID, item, **kwargs): ...
    def _CatalogRenderGlobalsStatsWidget__catalog_renderStatsUpdate_CB(self, eventType, eventID, item, **kwargs): ...
    def getStats(self) -> PyXmlIO.Element | None: ...
    def setCatalogItem(self, item): ...
    def updateState(self): ...

class CatalogRenderTaskByNameStatsWidget(PyQt5.QtWidgets.QTreeWidget):
    statsPresent: ClassVar[PyQt5.QtCore.pyqtSignal] = ...
    def __init__(self, parent: Incomplete | None = ...) -> None: ...
    def _CatalogRenderTaskByNameStatsWidget__catalog_itemDelete_CB(self, eventType, eventID, item, **kwargs): ...
    def _CatalogRenderTaskByNameStatsWidget__catalog_renderStatsUpdate_CB(self, eventType, eventID, item, **kwargs): ...
    def getStats(self) -> PyXmlIO.Element | None: ...
    def setCatalogItem(self, item): ...
    def updateState(self): ...

class CatalogRenderTaskByTypeStatsWidget(PyQt5.QtWidgets.QTreeWidget):
    statsPresent: ClassVar[PyQt5.QtCore.pyqtSignal] = ...
    def __init__(self, parent: Incomplete | None = ...) -> None: ...
    def _CatalogRenderTaskByTypeStatsWidget__catalog_itemDelete_CB(self, eventType, eventID, item, **kwargs): ...
    def _CatalogRenderTaskByTypeStatsWidget__catalog_renderStatsUpdate_CB(self, eventType, eventID, item, **kwargs): ...
    def getStats(self) -> PyXmlIO.Element | None: ...
    def setCatalogItem(self, item): ...
    def updateState(self): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import Utils.EventModule as EventModule
import QT4GLLayerStack as QT4GLLayerStack
import PyQt5.QtCore as QtCore
import Utils as Utils
from PyUtilModule.VirtualKatana import RenderGlobals as RenderGlobals, RenderManager as RenderManager
from QT4GLLayerStack.RectangleLayer import RectangleLayer
from typing import Set, Tuple

class RoiLayer(RectangleLayer):
    def __init__(self, valuePolicy, *args, **kwargs) -> None: ...
    def _RoiLayer__renderManager_roiChanged_CB(self, eventType, eventID, *args, **kwargs): ...
    def _RoiLayer__updateROI(self): ...
    def getDataWindow(self): ...
    def getDisplayWindow(self): ...
    def processEvent(self, event): ...
    def setRectangle(self, rect): ...
    def update(self): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import Callbacks as Callbacks
import PackageSuperToolAPI.NodeUtils as NU
import Naming as Naming
import NodegraphAPI as NodegraphAPI
import Utils as Utils
import typing
from Callbacks.Callbacks import _TypeEnum
from PyUtilModule.VirtualKatana import Decorators as Decorators
from Utils.Decorators import deprecated as deprecated
from _typeshed import Incomplete
from typing import ClassVar, Set, Tuple

__AdoptableLocationTypes: dict
__DeprecatedPackageClasses: dict
__DisplayPackageClasses: dict
__GroupTypeRegistry: dict
__PackageClassRegistry: dict

class CallbackMixin(Mixin):
    _CallbackMixin__shaderSelectedCallbackRegistered: ClassVar[bool] = ...
    _OnGafferShaderSelectedCallbackType: ClassVar[str] = ...
    _OnPackageCreatedCallbackID: ClassVar[None] = ...
    _OnShaderSelectedCallbackID: ClassVar[_TypeEnum] = ...
    @classmethod
    def executeCreationCallback(cls, package: Package): ...
    @classmethod
    def onSceneGraphViewShaderSelected(cls, objectHash: Incomplete | None = ..., node: typing.Optional[NodegraphAPI.Node] = ..., parameterPolicy: Incomplete | None = ...): ...
    @classmethod
    def registerCallbacks(cls, packageClass: type): ...

class DisableableMixin(Mixin):
    def _DisableableMixin__createDisableNodes(self) -> NodegraphAPI.Node: ...
    def isDisabled(self) -> bool: ...
    def isDisabledByParents(self) -> bool: ...
    def setDisabled(self, disabled: bool): ...

class DummyGroupPackage(GroupPackage):
    DEFAULT_NAME: ClassVar[str] = ...
    DISPLAY_ICON: ClassVar[str] = ...
    def __init__(self, packageNode: NodegraphAPI.Node | None, locationPath: Incomplete | None = ..., mainNode: Incomplete | None = ...) -> None: ...
    def canAdoptPackage(self, package: Package) -> bool: ...
    @classmethod
    def canCreateChildPackage(cls, packageClassOrName: type | str, mainNode: NodegraphAPI.Node, locationPath: str) -> bool: ...
    def childRemoved(self): ...
    @classmethod
    def create(cls, enclosingNode, locationPath): ...
    def createChildPackage(self): ...
    @classmethod
    def createForLocation(cls, locationPath: str, mainNode: NodegraphAPI.Node): ...
    def createParentPackageHierarchy(self): ...
    def delete(self): ...
    def getChildPackageCreateNode(self) -> NodegraphAPI.Node: ...
    def getChildPackages(self) -> list[Package]: ...
    def getLocationPath(self) -> str: ...
    def getMainNode(self) -> NodegraphAPI.Node: ...
    def getParentPackage(self) -> Package | None: ...

class EditPackage(Package):
    def canBeAdoptedByPackage(self, package: Package) -> bool: ...
    @classmethod
    def canBeCreatedByPackageClass(cls, packageClass: type) -> bool: ...
    def canBeRenamed(self) -> bool: ...
    @classmethod
    def canCreatePackageClass(cls, packageClass: type) -> bool: ...
    @classmethod
    def create(cls, enclosingNode: NodegraphAPI.Node, locationPath: str) -> Package: ...
    @classmethod
    def createPackage(cls, packageNode: NodegraphAPI.Node) -> Package: ...
    @classmethod
    def createPackageEditStackNode(cls, enclosingNode: NodegraphAPI.Node, locationPath: str) -> NodegraphAPI.Node: ...
    def delete(self): ...

class GroupEditPackage(GroupMixin, EditPackage):
    def _GroupEditPackage__getCreateStackPackage(self, create: bool = ...) -> Package | None: ...
    def childRemoved(self): ...
    @classmethod
    def create(cls, enclosingNode: NodegraphAPI.Node, locationPath: str) -> Package: ...
    def getChildPackageCreateNode(self) -> NodegraphAPI.Node: ...
    def getChildPackages(self) -> list[Package]: ...

class GroupMixin(Mixin):
    def adoptPackage(self, childPackage: Package, peerNames: Incomplete | None = ...): ...
    def canAdoptPackage(self, package: Package) -> bool: ...
    @classmethod
    def canCreateChildPackage(cls, packageClassOrName: type | str, mainNode: NodegraphAPI.Node, locationPath: str) -> bool: ...
    def canReorderChildPackage(self, oldIndex: int, newIndex: int) -> bool: ...
    def childRemoved(self): ...
    def createChildPackage(self, packageClassOrName: type | str, name: Incomplete | None = ..., peerNames: Incomplete | None = ...) -> Package | None: ...
    def getChildPackage(self, name: str, includeDummies: bool = ...) -> Package | None: ...
    def getChildPackageCreateNode(self) -> NodegraphAPI.Node: ...
    def getChildPackages(self) -> list[Package]: ...
    @classmethod
    def getLocationTypes(cls) -> list[str]: ...
    def reorderChildPackage(self, oldIndex: int, newIndex: int): ...

class GroupPackage(GroupMixin, Package):
    DEFAULT_NAME: ClassVar[str] = ...
    DISPLAY_ICON: ClassVar[str] = ...
    @classmethod
    def create(cls, enclosingNode: NodegraphAPI.Node, locationPath: str) -> Package: ...
    @classmethod
    def createStandardPackageNodes(cls, packageNode: NodegraphAPI.Node) -> list[NodegraphAPI.Node]: ...
    def getChildPackages(self) -> list[Package]: ...
    def getExtraNodeDependencies(self) -> list[NodegraphAPI.Node]: ...
    def initializeExtraNodeDependencies(self): ...

class LinkingMixin(Mixin):
    LINKING_ILLUMINATION_EFFECT: ClassVar[str] = ...
    LINKING_ILLUMINATION_NODE_NAME: ClassVar[str] = ...
    LINKING_ILLUMINATION_NODE_REF: ClassVar[str] = ...
    LINKING_SHADOW_EFFECT: ClassVar[str] = ...
    LINKING_SHADOW_NODE_NAME: ClassVar[str] = ...
    LINKING_SHADOW_NODE_REF: ClassVar[str] = ...
    @classmethod
    def _createLinkingNode(cls, packageNode: NodegraphAPI.Node, nodeName: str, nodeRef: str, effect: str) -> NodegraphAPI.Node: ...
    @classmethod
    def getIlluminationLinkingNode(cls, packageNode: NodegraphAPI.Node, create: bool = ...) -> NodegraphAPI.Node | None: ...
    @classmethod
    def getLinkingNodes(cls, packageNode: NodegraphAPI.Node, create: bool = ...) -> Tuple[NodegraphAPI.Node, NodegraphAPI.Node] | Tuple[None, None]: ...
    @classmethod
    def getShadowLinkingNode(cls, packageNode: NodegraphAPI.Node, create: bool = ...) -> NodegraphAPI.Node | None: ...

class LockingMixin(Mixin):
    def isLocked(self) -> bool: ...
    def setLocked(self, state: bool): ...

class LookFileReferenceEditMixin(Mixin):
    def _getLookFileReferenceNode(self) -> NodegraphAPI.Node | None: ...
    def _getLookFileReferenceNodeTypeName(self) -> str | None: ...
    def _getOrCreateLookFileReferenceNode(self, forceCreate: bool) -> Tuple[NodegraphAPI.Node, NodegraphAPI.Node] | Tuple[None, None]: ...
    def getLookFileMaterial(self) -> None | None: ...
    def isLookFileMaterialActive(self) -> bool: ...
    def setLookFileMaterial(self, asset: str | None, materialPath: str | None, assetAsExpression: bool = ..., materialPathAsExpression: bool = ...): ...
    def setLookFileMaterialActive(self, lookFileMaterialActive: bool): ...

class MaterialMixin(Mixin):
    _OnLookFileMaterialActiveSetCallbackID: ClassVar[_TypeEnum] = ...
    _OnLookFileMaterialSetCallbackID: ClassVar[_TypeEnum] = ...
    def getLookFileMaterial(self) -> tuple[str, str] | Tuple[None, None]: ...
    def getShader(self, shaderType: str) -> str | None: ...
    def isLookFileMaterialActive(self) -> bool: ...
    def isUsingLookFileMaterial(self): ...
    def setIsUsingLookFileMaterial(self, newValue): ...
    def setLookFileMaterial(self, asset: str | None, materialPath: str, assetAsExpression: bool = ..., materialPathAsExpression: bool = ...): ...
    def setLookFileMaterialActive(self, lookFileMaterialActive: bool): ...
    def setShader(self, shaderType: str, shaderName: str | None, asExpression: bool = ...): ...

class Mixin(Upgradable): ...

class MuteAndSoloEditMixin(MuteAndSoloMixin):
    def delete(self): ...
    def setMuted(self, muted: bool): ...
    def setSoloed(self, soloed: bool): ...

class MuteAndSoloMixin(Mixin):
    MUTEANDSOLO_NODE_REFERENCE: ClassVar[str] = ...
    NODE_NAME: ClassVar[str] = ...
    SOLOLISTEDIT_NODE_REFERENCE: ClassVar[str] = ...
    @classmethod
    def _createMuteAndSolo(cls, packageNode: NodegraphAPI.Node): ...
    @classmethod
    def _createSoloListEdit(cls, packageNode: NodegraphAPI.Node) -> NodegraphAPI.Node: ...
    def _destroyMuteSoloNodeIfUnused(self, muteAndSoloNode: NodegraphAPI.Node, soloListEditNode: NodegraphAPI.Node): ...
    def _getState(self, node: NodegraphAPI.Node, name: str, default: bool = ...) -> bool: ...
    def _setState(self, node: NodegraphAPI.Node, name: str, state: bool): ...
    def delete(self, edit: bool = ...): ...
    def getExtraNodeDependencies(self) -> list[NodegraphAPI.Node]: ...
    def getMuteSoloAndSoloListEditNode(self, create: bool = ...): ...
    def getOverrideNodeAndParameter(self, attributeName: str) -> Tuple[NodegraphAPI.Node, NodegraphAPI.Parameter] | Tuple[None, None]: ...
    def initializeExtraNodeDependencies(self): ...
    def isMuteOverrideEnabled(self) -> bool: ...
    def isMuted(self) -> bool: ...
    def isSoloOverrideEnabled(self) -> bool: ...
    def isSoloed(self) -> bool: ...
    def setMuted(self, muted: bool, edit: bool = ...): ...
    def setSoloed(self, soloed: bool, edit: bool = ...): ...

class Package(Upgradable):
    _Package__editPackageClasses: ClassVar[dict] = ...
    def __init__(self, packageNode: NodegraphAPI.Node) -> None: ...
    def _setPackageNode(self, packageNode: NodegraphAPI.Node): ...
    def adoptPackage(self, childPackage: Package, peerNames: Incomplete | None = ...): ...
    def canAdoptPackage(self, package: Package) -> bool: ...
    def canBeAdoptedByPackage(self, package: Package) -> bool: ...
    @classmethod
    def canBeCreatedByPackageClass(cls, packageClass: type) -> bool: ...
    def canBeDeleted(self) -> bool: ...
    def canBeDuplicated(self, parentPackage: Incomplete | None = ...) -> bool: ...
    def canBeRenamed(self) -> bool: ...
    @classmethod
    def canCreateChildPackage(cls, packageClassOrName: type | str, mainNode: NodegraphAPI.Node, locationPath: str) -> bool: ...
    @classmethod
    def canCreatePackageClass(cls, packageClass: type) -> bool: ...
    def canDuplicate(self): ...
    def canReorderChildPackage(self, oldIndex: int, newIndex: int) -> bool: ...
    @classmethod
    def create(cls, enclosingNode: NodegraphAPI.Node, locationPath: str) -> Package: ...
    def createChildPackage(self, packageClassOrName: type | str, name: Incomplete | None = ..., peerNames: Incomplete | None = ...) -> Package | None: ...
    @classmethod
    def createPackage(cls, packageNode: NodegraphAPI.Node) -> Package: ...
    @classmethod
    def createPackageGroupNode(cls, enclosingNode: NodegraphAPI.Node, locationPath: str) -> NodegraphAPI.GroupNode: ...
    def createPostMergeStackNode(self) -> NodegraphAPI.GroupStack: ...
    @classmethod
    def createStandardPackageNodes(cls, packageNode: NodegraphAPI.Node) -> list[NodegraphAPI.Node]: ...
    def delete(self): ...
    def duplicate(self, peerNames: Incomplete | None = ..., parentPackage: Incomplete | None = ...) -> Package: ...
    @classmethod
    def getAdoptableLocationTypes(cls) -> set[str]: ...
    def getChildPackage(self, name: str, includeDummies: bool = ...) -> Package | None: ...
    def getChildPackages(self) -> list[Package]: ...
    def getCreateNode(self) -> NodegraphAPI.Node: ...
    @classmethod
    def getEditPackageClass(cls) -> type: ...
    def getExtraNodeDependencies(self) -> list[NodegraphAPI.Node]: ...
    def getLocationPath(self) -> str: ...
    def getMainNode(self) -> NodegraphAPI.Node: ...
    @classmethod
    def getMainNodeFromNode(cls, node: NodegraphAPI.Node) -> NodegraphAPI.Node | None: ...
    def getName(self) -> str: ...
    def getOrCreateNodeByType(self, nodeType: str, forceCreate: bool = ...) -> NodegraphAPI.Node | None: ...
    def getOverrideNodeAndParameter(self, attrName: str) -> Tuple[NodegraphAPI.Node, NodegraphAPI.Parameter]: ...
    @classmethod
    def getPackageClassFromNode(cls, node: NodegraphAPI.Node) -> Package | None: ...
    @classmethod
    def getPackageFromNode(cls, node: NodegraphAPI.Node) -> Package: ...
    def getPackageNode(self) -> NodegraphAPI.Node: ...
    def getParentPackage(self) -> Package | None: ...
    def getPostMergePackageStack(self, create: bool = ...) -> NodegraphAPI.GroupStack | None: ...
    def initializeExtraNodeDependencies(self): ...
    def isNodeOfType(self, node: NodegraphAPI.Node | None, nodeType: str) -> bool: ...
    def reorderChildPackage(self, oldIndex: int, newIndex: int): ...
    @classmethod
    def setEditPackageClass(cls, editPackageClass: type): ...
    def setName(self, name: str, queueEvent: bool = ..., selectLocation: bool = ..., makeUnique: bool = ..., replaceSelection: bool = ..., **kwargs) -> str: ...
    def supportsLocking(self) -> bool: ...
    @classmethod
    def walkUpPackageHierarchy(cls, leafPackage: Package, includeLeaf: bool = ...) -> typing.Iterator: ...
    def __eq__(self, other) -> bool: ...
    def __hash__(self) -> int: ...
    def __ne__(self, other) -> bool: ...

class RootPackage(GroupPackage):
    def __init__(self, packageNode: NodegraphAPI.Node) -> None: ...
    def canBeDeleted(self) -> bool: ...
    def canBeRenamed(self) -> bool: ...
    @classmethod
    def canCreatePackageClass(cls, packageClass: type) -> bool: ...
    @classmethod
    def create(cls, enclosingNode: NodegraphAPI.Node, locationPath: str) -> Package: ...
    def delete(self): ...
    def setName(self, name: str, peerNames: Incomplete | None = ..., queueEvent: bool = ..., selectLocation: bool = ..., replaceSelection: bool = ...): ...

class Upgradable:
    def upgradeToVersion(self, version: int): ...

def GetPackageClass(superToolName: str | None, packageClassName: str) -> Package | None: ...
def GetPackageClassAndPackageName(superToolName: str, packageClassOrName: type | str) -> Tuple[type, str] | Tuple[None, None]: ...
def GetPackageClassForGroupLocationType(superToolName: str | None, groupLocationType: str) -> Package | None: ...
def GetRegisteredDisplayPackageClasses(superToolName: str | None) -> list[Package]: ...
def GetRegisteredPackageClasses(superToolName: str | None) -> list[str, type]: ...
def IsLocationTypeAdoptable(superToolName: str | None, locationType: str) -> bool: ...
def RegisterDeprecatedPackageClass(superToolName: str | None, deprecatedPackageClassName: str, packageClass: type): ...
def RegisterPackage(superToolName, packageClass): ...
def RegisterPackageClass(superToolName: str | None, packageClass: type): ...
